import base64
import uuid
from datetime import datetime, timedelta
import dateutil.parser as dateparser

from marshmallow import post_load, fields
from sqlalchemy import orm, and_, or_, func

from core.managers.db_manager import db
from core.managers.log_manager import logger
from core.model.acl_entry import ACLEntry
from core.model.osint_source import OSINTSourceGroup, OSINTSource
from shared.schema.acl_entry import ItemType
from shared.schema.news_item import (
    NewsItemDataSchema,
    NewsItemAggregateSchema,
    NewsItemAttributeSchema,
    NewsItemSchema,
    NewsItemRemoteSchema,
)


class NewNewsItemAttributeSchema(NewsItemAttributeSchema):
    @post_load
    def make(self, data, **kwargs):
        return NewsItemAttribute(**data)


class NewNewsItemDataSchema(NewsItemDataSchema):
    attributes = fields.Nested(NewNewsItemAttributeSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return NewsItemData(**data)


news_item_data_news_item_tag = db.Table(
    db.Column(
        "news_item_data_id",
        db.String,
        db.ForeignKey("news_item_data.id"),
        primary_key=True,
    ),
    db.Column(
        "news_item_tag_id",
        db.Integer,
        db.ForeignKey("news_item_tag.id"),
        primary_key=True,
    ),
)


class NewsItemData(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    hash = db.Column(db.String())

    title = db.Column(db.String())
    review = db.Column(db.String())
    author = db.Column(db.String())
    source = db.Column(db.String())
    link = db.Column(db.String())
    language = db.Column(db.String())
    content = db.Column(db.String())
    collected = db.Column(db.DateTime)
    published = db.Column(db.String())
    updated = db.Column(db.DateTime, default=datetime.now())

    attributes = db.relationship("NewsItemAttribute", secondary="news_item_data_news_item_attribute")

    osint_source_id = db.Column(db.String, db.ForeignKey("osint_source.id"), nullable=True)
    osint_source = db.relationship("OSINTSource")
    remote_source = db.Column(db.String())

    def __init__(
        self,
        id,
        hash,
        title,
        review,
        source,
        link,
        published,
        author,
        collected,
        content,
        osint_source_id,
        attributes,
    ):
        self.id = str(uuid.uuid4()) if id is None else id
        self.hash = hash
        self.title = title
        self.review = review
        self.source = source
        self.link = link
        self.published = published
        self.author = author
        self.collected = collected
        self.content = content
        self.attributes = attributes
        self.osint_source_id = osint_source_id

    @classmethod
    def allowed_with_acl(cls, news_item_data_id, user, see, access, modify):

        news_item_data = cls.query.get(news_item_data_id)
        if news_item_data.remote_source is not None:
            return True
        query = db.session.query(NewsItemData.id).distinct().group_by(NewsItemData.id).filter(NewsItemData.id == news_item_data_id)

        query = query.join(OSINTSource, NewsItemData.osint_source_id == OSINTSource.id)

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    NewsItemData.osint_source_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.OSINT_SOURCE,
                ),
                and_(
                    OSINTSource.collector_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.COLLECTOR,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, see, access, modify)

        return query.scalar() is not None

    @classmethod
    def identical(cls, hash):
        return db.session.query(db.exists().where(NewsItemData.hash == hash)).scalar()

    @classmethod
    def find_by_hash(cls, hash):
        return cls.query.filter(NewsItemData.hash == hash).all()

    @classmethod
    def count_all(cls):
        return cls.query.count()

    @classmethod
    def latest_collected(cls):
        news_item_data = cls.query.order_by(db.desc(NewsItemData.collected)).limit(1).all()
        if len(news_item_data) > 0:
            return news_item_data[0].collected.strftime("%d.%m.%Y - %H:%M")
        else:
            return ""

    @classmethod
    def get_all_news_items_data(cls, limit):
        limit = datetime.strptime(limit, "%d.%m.%Y - %H:%M")
        limit = datetime.strftime(limit, "%Y-%m-%d - %H:%M")

        news_items_data = cls.query.filter(cls.collected > limit).all()
        news_items_data_schema = NewsItemDataSchema(many=True)
        return news_items_data_schema.dump(news_items_data)

    @classmethod
    def attribute_value_identical(cls, id, value):
        return (
            NewsItemAttribute.query.join(NewsItemDataNewsItemAttribute)
            .join(NewsItemData)
            .filter(NewsItemData.id == id)
            .filter(NewsItemAttribute.value == value)
            .scalar()
        )

    @classmethod
    def update_news_item_attributes(cls, news_item_id, attributes):
        news_item = cls.query.filter_by(id=news_item_id).first()

        attributes_schema = NewNewsItemAttributeSchema(many=True)
        attributes = attributes_schema.load(attributes)

        for attribute in attributes:
            if not cls.attribute_value_identical(news_item_id, attribute.value):
                news_item.attributes.append(attribute)
                db.session.commit()

    @classmethod
    def get_news_item_data(cls, news_item_id):
        query = cls.query.join(NewsItem, NewsItemData.id == NewsItem.news_item_data_id)
        query = query.filter(NewsItem.id == news_item_id)
        return query

    @classmethod
    def update_news_item_tags(cls, news_item_id, tags):
        try:
            n_i_d = NewsItemData.get_news_item_data(news_item_id).first()
            n_i_d.tags = tags
            db.session.commit()
        except Exception:
            logger.log_debug_trace("Update News Item Tags Failed")

    @classmethod
    def get_for_sync(cls, last_synced, osint_sources):
        osint_source_ids = set()
        for osint_source in osint_sources:
            osint_source_ids.add(osint_source.id)

        last_sync_time = datetime.now()

        query = cls.query.filter(
            NewsItemData.updated >= last_synced,
            NewsItemData.updated <= last_sync_time,
            NewsItemData.osint_source_id.in_(osint_source_ids),
        )

        news_items = query.all()
        news_item_remote_schema = NewsItemRemoteSchema(many=True)
        for news_item in news_items:
            total_relevance = NewsItem.get_total_relevance(news_item.id)
            if total_relevance > 0:
                news_item.relevance = 1
            elif total_relevance < 0:
                news_item.relevance = -1
            else:
                news_item.relevance = 0

        items = news_item_remote_schema.dump(news_items)

        return items, last_sync_time


class NewsItemTag(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255))
    tag_type = db.Column(db.String(255))
    n_i_d_id = db.Column(db.ForeignKey(NewsItemData.id), nullable=False)
    n_i_d = db.relationship(NewsItemData, backref="tags")


class NewsItem(db.Model):
    id = db.Column(db.Integer, primary_key=True)

    read = db.Column(db.Boolean, default=False)
    important = db.Column(db.Boolean, default=False)

    likes = db.Column(db.Integer, default=0)
    dislikes = db.Column(db.Integer, default=0)
    relevance = db.Column(db.Integer, default=0)

    news_item_data_id = db.Column(db.String, db.ForeignKey("news_item_data.id"))
    news_item_data = db.relationship("NewsItemData")

    news_item_aggregate_id = db.Column(db.Integer, db.ForeignKey("news_item_aggregate.id"))

    @classmethod
    def find(cls, news_item_id):
        return cls.query.get(news_item_id)

    @classmethod
    def get_all_with_data(cls, news_item_data_id):
        return cls.query.filter_by(news_item_data_id=news_item_data_id).all()

    @classmethod
    def get_detail_json(cls, id):
        news_item = cls.query.get(id)
        news_item_schema = NewsItemSchema()
        return news_item_schema.dump(news_item)

    @classmethod
    def get_by_group(cls, group_id, filter, offset, limit, user):
        query = cls.query.distinct().group_by(NewsItem.id)
        query = query.join(NewsItemData, NewsItem.news_item_data_id == NewsItemData.id)
        query = query.outerjoin(OSINTSource, NewsItemData.osint_source_id == OSINTSource.id)
        query = query.filter(OSINTSource.id == group_id)

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    NewsItemData.osint_source_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.OSINT_SOURCE,
                ),
                and_(
                    OSINTSource.collector_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.COLLECTOR,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, True, False, False)

        if "search" in filter and filter["search"] != "":
            search_string = f"%{filter['search'].lower()}%"
            query = query.filter(NewsItemData.content.like(search_string))

        if "read" in filter and filter["read"].lower() == "true":
            query = query.filter(NewsItem.read is False)

        if "important" in filter and filter["important"].lower() == "true":
            query = query.filter(NewsItem.important is True)

        if "relevant" in filter and filter["relevant"].lower() == "true":
            query = query.filter(NewsItem.likes > 0)

        if "range" in filter and filter["range"] != "ALL":
            date_limit = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

            if filter["range"] == "WEEK":
                date_limit -= timedelta(days=date_limit.weekday())

            elif filter["range"] == "MONTH":
                date_limit = date_limit.replace(day=1)

            query = query.filter(NewsItemData.collected >= date_limit)

        if "sort" in filter:
            if filter["sort"] == "DATE_DESC":
                query = query.order_by(db.desc(NewsItemData.collected), db.desc(NewsItemAggregate.id))

            elif filter["sort"] == "DATE_ASC":
                query = query.order_by(db.asc(NewsItemData.collected), db.asc(NewsItemAggregate.id))

            elif filter["sort"] == "RELEVANCE_DESC":
                query = query.order_by(db.desc(NewsItem.relevance), db.desc(NewsItem.id))

            elif filter["sort"] == "RELEVANCE_ASC":
                query = query.order_by(db.asc(NewsItem.relevance), db.asc(NewsItem.id))

        return query.offset(offset).limit(limit).all(), query.count()

    @classmethod
    def get_by_group_json(cls, group_id, filter, offset, limit, user):
        news_items, count = cls.get_by_group(group_id, filter, offset, limit, user)

        for news_item in news_items:
            see, access, modify = NewsItem.get_acl_status(news_item.id, user)
            news_item.see = see
            news_item.access = access
            news_item.modify = modify
            vote = NewsItemVote.find(news_item.id, user.id)
            if vote is not None:
                news_item.me_like = vote.like
                news_item.me_dislike = vote.dislike
            else:
                news_item.me_like = False
                news_item.me_dislike = False

        news_item_schema = NewsItemSchema(many=True)
        items = news_item_schema.dump(news_items)

        return {"total_count": count, "items": items}

    @classmethod
    def get_total_relevance(cls, news_item_data_id):
        query = db.session.query(NewsItem.relevance).filter(NewsItem.news_item_data_id == news_item_data_id)
        result = query.all()
        return sum(int(row[0]) for row in result)

    @classmethod
    def get_all_by_group_and_source_query(cls, group_id, source_id, time_limit):
        query = cls.query.join(NewsItemData, NewsItemData.id == NewsItem.news_item_data_id)
        query = query.filter(
            NewsItemData.osint_source_id == source_id,
            NewsItemData.collected >= time_limit,
        )
        query = query.join(NewsItemAggregate, NewsItemAggregate.id == NewsItem.news_item_aggregate_id)
        query = query.filter(NewsItemAggregate.osint_source_group_id == group_id)
        return query

    @classmethod
    def allowed_with_acl(cls, news_item_id, user, see, access, modify):

        news_item = cls.query.get(news_item_id)
        if news_item.news_item_data.remote_source is not None:
            return True
        query = db.session.query(NewsItem.id).distinct().group_by(NewsItem.id).filter(NewsItem.id == news_item_id)

        query = query.join(NewsItemData, NewsItem.news_item_data_id == NewsItemData.id)
        query = query.join(OSINTSource, NewsItemData.osint_source_id == OSINTSource.id)

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    NewsItemData.osint_source_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.OSINT_SOURCE,
                ),
                and_(
                    OSINTSource.collector_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.COLLECTOR,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, see, access, modify)

        return query.scalar() is not None

    @classmethod
    def get_acl_status(cls, news_item_id, user):

        news_item = cls.query.get(news_item_id)
        if news_item.news_item_data.remote_source is not None:
            return True, True, True
        query = (
            db.session.query(
                NewsItem.id,
                func.count().filter(ACLEntry.id > 0).label("acls"),
                func.count().filter(ACLEntry.see).label("see"),
                func.count().filter(ACLEntry.access).label("access"),
                func.count().filter(ACLEntry.modify).label("modify"),
            )
            .distinct()
            .group_by(NewsItem.id)
            .filter(NewsItem.id == news_item_id)
        )

        query = query.join(NewsItemData, NewsItem.news_item_data_id == NewsItemData.id)
        query = query.outerjoin(OSINTSource, NewsItemData.osint_source_id == OSINTSource.id)

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    NewsItemData.osint_source_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.OSINT_SOURCE,
                ),
                and_(
                    OSINTSource.collector_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.COLLECTOR,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, False, False, False)

        result = query.all()
        see = result[0].see > 0 or result[0].acls == 0
        access = result[0].access > 0 or result[0].acls == 0
        modify = result[0].modify > 0 or result[0].acls == 0

        return see, access, modify

    def vote(self, data, user_id):
        if "vote" not in data:
            return
        self.news_item_data.updated = datetime.now()
        vote = NewsItemVote.find(self.id, user_id)
        if vote is None:
            vote = NewsItemVote(self.id, user_id)
            db.session.add(vote)

        if data["vote"] > 0:
            if vote.like is True:
                self.likes -= 1
                self.relevance -= 1
                vote.like = False
            else:
                self.likes += 1
                self.relevance += 1
                vote.like = True
                if vote.dislike is True:
                    self.dislikes -= 1
                    self.relevance += 1
                    vote.dislike = False
        elif vote.dislike is True:
            self.dislikes -= 1
            self.relevance += 1
            vote.dislike = False
        else:
            self.dislikes += 1
            self.relevance -= 1
            vote.dislike = True
            if vote.like is True:
                self.likes -= 1
                self.relevance -= 1
                vote.like = False

    @classmethod
    def update(cls, id, data, user_id):
        news_item = cls.find(id)

        news_item.update_status(data, user_id)

        NewsItemAggregate.update_status(news_item.news_item_aggregate_id)
        db.session.commit()

        if "vote" in data:
            return "success", {news_item.news_item_data.osint_source_id}, 200
        else:
            return "success", {}, 200

    def update_status(self, data, user_id):
        self.vote(data, user_id)

        if "read" in data:
            self.read = not self.read

        if "important" in data:
            self.important = not self.important

    @classmethod
    def delete(cls, news_item_id):
        news_item = cls.find(news_item_id)
        if NewsItemAggregate.action_allowed([{"type": "AGGREGATE", "id": news_item.news_item_aggregate_id}]) is False:
            return "aggregate_in_use", 500
        aggregate_id = news_item.news_item_aggregate_id
        aggregate = NewsItemAggregate.find(aggregate_id)
        aggregate.news_items.remove(news_item)
        NewsItemVote.delete_all(news_item_id)
        db.session.delete(news_item)
        NewsItemAggregate.update_status(aggregate_id)
        db.session.commit()

        return "success", 200

    @classmethod
    def delete_only(cls, news_item):
        NewsItemVote.delete_all(news_item.id)
        db.session.delete(news_item)


class NewsItemVote(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    like = db.Column(db.Boolean)
    dislike = db.Column(db.Boolean)
    news_item_id = db.Column(db.Integer, db.ForeignKey("news_item.id"))
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), nullable=True)

    remote_node_id = db.Column(db.Integer, db.ForeignKey("remote_node.id"), nullable=True)
    remote_user = db.Column(db.String())

    def __init__(self, news_item_id, user_id):
        self.id = None
        self.news_item_id = news_item_id
        self.user_id = user_id
        self.like = False
        self.dislike = False

    @classmethod
    def find(cls, news_item_id, user_id):
        return cls.query.filter_by(news_item_id=news_item_id, user_id=user_id).first()

    @classmethod
    def delete_all(cls, news_item_id):
        votes = cls.query.filter_by(news_item_id=news_item_id).all()
        for vote in votes:
            db.session.delete(vote)

    @classmethod
    def delete_for_remote_node(cls, news_item_id, remote_node_id):
        vote = cls.query.filter_by(news_item_id=news_item_id, remote_node_id=remote_node_id).first()
        if vote is None:
            return 0
        db.session.delete(vote)
        return 1 if vote.like else -1


class NewsItemAggregate(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String())
    description = db.Column(db.String())

    created = db.Column(db.DateTime)

    read = db.Column(db.Boolean, default=False)
    important = db.Column(db.Boolean, default=False)

    likes = db.Column(db.Integer, default=0)
    dislikes = db.Column(db.Integer, default=0)
    relevance = db.Column(db.Integer, default=0)

    comments = db.Column(db.String(), default="")

    osint_source_group_id = db.Column(db.String, db.ForeignKey("osint_source_group.id"))

    news_items = db.relationship("NewsItem")

    news_item_attributes = db.relationship("NewsItemAttribute", secondary="news_item_aggregate_news_item_attribute")

    @classmethod
    def find(cls, news_item_aggregate_id):
        return cls.query.get(news_item_aggregate_id)

    @classmethod
    def get_by_group(cls, group_id, filter, offset, limit, user):
        query = cls.query.distinct().group_by(NewsItemAggregate.id)

        query = query.filter(NewsItemAggregate.osint_source_group_id == group_id)

        query = query.join(NewsItem, NewsItem.news_item_aggregate_id == NewsItemAggregate.id)
        query = query.join(NewsItemData, NewsItem.news_item_data_id == NewsItemData.id)
        query = query.outerjoin(OSINTSource, NewsItemData.osint_source_id == OSINTSource.id)

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    NewsItemData.osint_source_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.OSINT_SOURCE,
                ),
                and_(
                    OSINTSource.collector_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.COLLECTOR,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, True, False, False)

        if "search" in filter and filter["search"] != "":
            search_string = "%" + filter["search"].lower() + "%"
            query = query.join(
                NewsItemAggregateSearchIndex,
                NewsItemAggregate.id == NewsItemAggregateSearchIndex.news_item_aggregate_id,
            ).filter(NewsItemAggregateSearchIndex.data.like(search_string))

        if "read" in filter and filter["read"].lower() == "true":
            query = query.filter(NewsItemAggregate.read is False)

        if "important" in filter and filter["important"].lower() == "true":
            query = query.filter(NewsItemAggregate.important is True)

        if "relevant" in filter and filter["relevant"].lower() == "true":
            query = query.filter(NewsItemAggregate.likes > 0)

        if "in_analyze" in filter and filter["in_analyze"].lower() == "true":
            query = query.join(
                ReportItemNewsItemAggregate,
                NewsItemAggregate.id == ReportItemNewsItemAggregate.news_item_aggregate_id,
            )

        if "range" in filter and filter["range"] != "ALL":
            date_limit = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

            if filter["range"] == "WEEK":
                date_limit -= timedelta(days=date_limit.weekday())

            elif filter["range"] == "MONTH":
                date_limit = date_limit.replace(day=1)

            query = query.filter(NewsItemAggregate.created >= date_limit)

        if "sort" in filter:
            if filter["sort"] == "DATE_DESC":
                query = query.order_by(db.desc(NewsItemAggregate.created), db.desc(NewsItemAggregate.id))

            elif filter["sort"] == "DATE_ASC":
                query = query.order_by(db.asc(NewsItemAggregate.created), db.asc(NewsItemAggregate.id))

            elif filter["sort"] == "RELEVANCE_DESC":
                query = query.order_by(db.desc(NewsItemAggregate.relevance), db.desc(NewsItemAggregate.id))

            elif filter["sort"] == "RELEVANCE_ASC":
                query = query.order_by(db.asc(NewsItemAggregate.relevance), db.asc(NewsItemAggregate.id))

        return query.offset(offset).limit(limit).all(), query.count()

    @classmethod
    def get_by_group_json(cls, group_id, filter, offset, limit, user):
        news_item_aggregates, count = cls.get_by_group(group_id, filter, offset, limit, user)
        for news_item_aggregate in news_item_aggregates:
            news_item_aggregate.me_like = False
            news_item_aggregate.me_dislike = False
            for news_item in news_item_aggregate.news_items:
                see, access, modify = NewsItem.get_acl_status(news_item.id, user)
                news_item.see = see
                news_item.access = access
                news_item.modify = modify
                vote = NewsItemVote.find(news_item.id, user.id)
                if vote is not None:
                    news_item.me_like = vote.like
                    news_item.me_dislike = vote.dislike
                else:
                    news_item.me_like = False
                    news_item.me_dislike = False

                if news_item.me_like is True:
                    news_item_aggregate.me_like = True

                if news_item.me_dislike is True:
                    news_item_aggregate.me_dislike = True

            news_item_aggregate.in_reports_count = ReportItemNewsItemAggregate.count(news_item_aggregate.id)

        news_item_aggregate_schema = NewsItemAggregateSchema(many=True)
        items = news_item_aggregate_schema.dump(news_item_aggregates)
        for aggregate in items:
            for news_item in aggregate["news_items"][:]:
                if news_item["see"] is False:
                    aggregate["news_items"].remove(news_item)

        return {"total_count": count, "items": items}

    @classmethod
    def create_new_for_all_groups(cls, news_item_data):
        groups = OSINTSourceGroup.get_all_with_source(news_item_data.osint_source_id)
        for group in groups:
            news_item = NewsItem()
            news_item.news_item_data = news_item_data
            db.session.add(news_item)

            aggregate = NewsItemAggregate()
            aggregate.title = news_item_data.title
            aggregate.description = news_item_data.review
            aggregate.created = news_item_data.collected
            aggregate.osint_source_group_id = group.id
            aggregate.news_items.append(news_item)
            db.session.add(aggregate)

            NewsItemAggregateSearchIndex.prepare(aggregate)

    @classmethod
    def create_new_for_group(cls, news_item_data, osint_source_group_id):
        news_item = NewsItem()
        news_item.news_item_data = news_item_data
        db.session.add(news_item)

        aggregate = NewsItemAggregate()
        aggregate.title = news_item_data.title
        aggregate.description = news_item_data.review
        aggregate.created = news_item_data.collected
        aggregate.osint_source_group_id = osint_source_group_id
        aggregate.news_items.append(news_item)
        db.session.add(aggregate)

        NewsItemAggregateSearchIndex.prepare(aggregate)

    @classmethod
    def add_news_items(cls, news_items_data_list):
        news_item_data_schema = NewNewsItemDataSchema(many=True)
        news_items_data = news_item_data_schema.load(news_items_data_list)
        osint_source_ids = set()

        for news_item_data in news_items_data:
            if not NewsItemData.identical(news_item_data.hash):
                db.session.add(news_item_data)
                cls.create_new_for_all_groups(news_item_data)
                osint_source_ids.add(news_item_data.osint_source_id)

        db.session.commit()

        return osint_source_ids

    @classmethod
    def add_news_item(cls, news_item_data):
        news_item_data_schema = NewNewsItemDataSchema()
        news_item_data = news_item_data_schema.load(news_item_data)
        if not news_item_data.id:
            news_item_data.id = str(uuid.uuid4())
        if not news_item_data.hash:
            news_item_data.hash = news_item_data.id
        db.session.add(news_item_data)
        cls.create_new_for_all_groups(news_item_data)
        db.session.commit()

        return {news_item_data.osint_source_id}

    @classmethod
    def reassign_to_new_groups(cls, osint_source_id, default_group_id):
        time_limit = datetime.now() - timedelta(days=7)
        news_items_query = NewsItem.get_all_by_group_and_source_query(default_group_id, osint_source_id, time_limit)
        for news_item in news_items_query:
            news_item_data = news_item.news_item_data
            aggregate = NewsItemAggregate.find(news_item.news_item_aggregate_id)
            aggregate.news_items.remove(news_item)
            NewsItemVote.delete_all(news_item.id)
            db.session.delete(news_item)
            NewsItemAggregate.update_status(aggregate.id)
            cls.create_new_for_all_groups(news_item_data)
            db.session.commit()

    @classmethod
    def add_remote_news_items(cls, news_items_data_list, remote_node, osint_source_group_id):
        news_item_data_schema = NewNewsItemDataSchema(many=True)
        news_items_data = news_item_data_schema.load(news_items_data_list)

        news_item_data_ids = set()
        for news_item_data in news_items_data:
            news_item_data.remote_source = remote_node.name
            for attribute in news_item_data.attributes:
                attribute.remote_node_id = remote_node.id
                attribute.remote_user = remote_node.name

            original_news_item = NewsItemData.find_by_hash(news_item_data.hash)

            if original_news_item is None:
                db.session.add(news_item_data)
                cls.create_new_for_group(news_item_data, osint_source_group_id)
                news_item_data_ids.add(str(news_item_data.id))
            else:
                original_news_item.updated = datetime.now()
                for attribute in original_news_item.attributes[:]:
                    if attribute.remote_node_id == remote_node.id:
                        original_news_item.attributes.remove(attribute)
                        db.session.delete(attribute)

                original_news_item.attributes.extend(news_item_data.attributes)
                news_item_data_ids.add(str(original_news_item.id))

        db.session.commit()

        aggregate_ids = set()
        current_index = 0
        for news_item_data_id in news_item_data_ids:
            news_items = NewsItem.get_all_with_data(news_item_data_id)
            for news_item in news_items:
                had_relevance = NewsItemVote.delete_for_remote_node(news_item.id, remote_node.id)
                news_item.relevance -= had_relevance
                if had_relevance > 0:
                    news_item.likes -= 1
                elif had_relevance < 0:
                    news_item.dislikes -= 1

                if news_items_data_list[current_index]["relevance"] != 0:
                    vote = NewsItemVote(news_item.id, None)
                    vote.remote_node_id = remote_node.id
                    vote.remote_user = remote_node.name
                    if news_items_data_list[current_index]["relevance"] > 0:
                        vote.like = True
                        news_item.relevance += 1
                        news_item.likes += 1
                    else:
                        vote.dislike = True
                        news_item.relevance -= 1
                        news_item.dislikes += 1

                    db.session.add(vote)

                aggregate_ids.add(news_item.news_item_aggregate_id)

            current_index += 1

        db.session.commit()

        for aggregate_id in aggregate_ids:
            cls.update_status(aggregate_id)

        db.session.commit()

    @classmethod
    def update(cls, id, data, user):
        aggregate = cls.find(id)

        all_important = all(news_item.important is not False for news_item in aggregate.news_items)

        osint_source_ids = set()

        for news_item in aggregate.news_items:
            if NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                if "vote" in data:
                    osint_source_ids.add(news_item.news_item_data.osint_source_id)
                    news_item.vote(data, user.id)

                if "read" in data:
                    news_item.read = not aggregate.read

                if "important" in data:
                    news_item.important = not all_important

        if "title" in data:
            aggregate.title = data["title"]

        if "description" in data:
            aggregate.description = data["description"]

        if "comments" in data:
            aggregate.comments = data["comments"]

        NewsItemAggregate.update_status(aggregate.id)
        NewsItemAggregateSearchIndex.prepare(aggregate)

        db.session.commit()

        return "success", osint_source_ids, 200

    @classmethod
    def delete(cls, id, user):
        if cls.action_allowed([{"type": "AGGREGATE", "id": id}]) is False:
            return "aggregate_in_use", 500

        aggregate = cls.find(id)
        for news_item in aggregate.news_items:
            if NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                aggregate.news_items.remove(news_item)
                NewsItem.delete_only(news_item)

        NewsItemAggregate.update_status(aggregate.id)

        db.session.commit()

        return "success", 200

    @classmethod
    def action_allowed(cls, items):
        aggregate_ids = set()
        for item in items:
            if item["type"] == "AGGREGATE":
                aggregate_ids.add(item["id"])
            else:
                news_item = NewsItem.find(item["id"])
                aggregate_ids.add(news_item.news_item_aggregate_id)

        return all(ReportItemNewsItemAggregate.assigned(aggregate_id) is not True for aggregate_id in aggregate_ids)

    @classmethod
    def group_action(cls, data, user):
        if not cls.action_allowed(data["items"]):
            return "aggregate_in_use", 500

        if data["action"] == "GROUP":
            cls.group_aggregate(data["items"], user)
        elif data["action"] == "UNGROUP":
            cls.ungroup_aggregate(data["items"], user)
        return "success", 200

    @classmethod
    def group_action_delete(cls, data, user):
        if not cls.action_allowed(data["items"]):
            return "aggregate_in_use", 500
        processed_aggregates = set()
        for item in data["items"]:
            if item["type"] == "AGGREGATE":
                aggregate = NewsItemAggregate.find(item["id"])
                for news_item in aggregate.news_items:
                    if NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                        aggregate.news_items.remove(news_item)
                        NewsItem.delete_only(news_item)

                processed_aggregates.add(aggregate)
            else:
                news_item = NewsItem.find(item["id"])
                if NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                    aggregate = NewsItemAggregate.find(news_item.news_item_aggregate_id)
                    aggregate.news_items.remove(news_item)
                    NewsItem.delete_only(news_item)
                    processed_aggregates.add(aggregate)

        cls.update_aggregates(processed_aggregates)
        db.session.commit()
        return "", 200

    @classmethod
    def group_aggregate(cls, items, user):
        new_aggregate = NewsItemAggregate()
        processed_aggregates = set()
        group_id = None
        for item in items:
            if item["type"] == "AGGREGATE":
                aggregate = NewsItemAggregate.find(item["id"])
                group_id = aggregate.osint_source_group_id
                if new_aggregate.title is None:
                    new_aggregate.title = aggregate.title
                    new_aggregate.description = aggregate.description
                    new_aggregate.created = aggregate.created

                for news_item in aggregate.news_items[:]:
                    if user is None or NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                        new_aggregate.news_items.append(news_item)
                        aggregate.news_items.remove(news_item)

                processed_aggregates.add(aggregate)
            else:
                news_item = NewsItem.find(item["id"])
                if user is None or NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                    aggregate = NewsItemAggregate.find(news_item.news_item_aggregate_id)
                    group_id = aggregate.osint_source_group_id
                    if new_aggregate.title is None:
                        new_aggregate.title = news_item.news_item_data.title
                        new_aggregate.description = news_item.news_item_data.review
                        new_aggregate.created = news_item.news_item_data.collected
                    new_aggregate.news_items.append(news_item)
                    aggregate.news_items.remove(news_item)
                    processed_aggregates.add(aggregate)

        new_aggregate.osint_source_group_id = group_id
        db.session.add(new_aggregate)
        db.session.commit()

        NewsItemAggregate.update_status(new_aggregate.id)
        NewsItemAggregateSearchIndex.prepare(new_aggregate)

        cls.update_aggregates(processed_aggregates)

        db.session.commit()

    @classmethod
    def ungroup_aggregate(cls, items, user):
        processed_aggregates = set()
        for item in items:
            if item["type"] == "AGGREGATE":
                aggregate = NewsItemAggregate.find(item["id"])
                group_id = aggregate.osint_source_group_id
                for news_item in aggregate.news_items[:]:
                    if NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                        aggregate.news_items.remove(news_item)
                        cls.create_single_aggregate(news_item, group_id)

                processed_aggregates.add(aggregate)
            else:
                news_item = NewsItem.find(item["id"])
                if NewsItem.allowed_with_acl(news_item.id, user, False, False, True):
                    aggregate = NewsItemAggregate.find(news_item.news_item_aggregate_id)
                    group_id = aggregate.osint_source_group_id
                    aggregate.news_items.remove(news_item)
                    processed_aggregates.add(aggregate)
                    cls.create_single_aggregate(news_item, group_id)

        cls.update_aggregates(processed_aggregates)

        db.session.commit()

    @classmethod
    def update_aggregates(cls, aggregates):
        for aggregate in aggregates:
            if len(aggregate.news_items) == 0:
                NewsItemAggregateSearchIndex.remove(aggregate)
                db.session.delete(aggregate)
            else:
                NewsItemAggregateSearchIndex.prepare(aggregate)
                NewsItemAggregate.update_status(aggregate.id)

    @classmethod
    def create_single_aggregate(cls, news_item, group_id):
        new_aggregate = NewsItemAggregate()
        new_aggregate.title = news_item.news_item_data.title
        new_aggregate.description = news_item.news_item_data.review
        new_aggregate.created = news_item.news_item_data.collected
        new_aggregate.news_items.append(news_item)
        new_aggregate.osint_source_group_id = group_id
        db.session.add(new_aggregate)
        db.session.commit()

        NewsItemAggregateSearchIndex.prepare(new_aggregate)
        NewsItemAggregate.update_status(new_aggregate.id)

    @classmethod
    def update_status(cls, aggregate_id):
        aggregate = cls.find(aggregate_id)

        if len(aggregate.news_items) == 0:
            NewsItemAggregateSearchIndex.remove(aggregate)
            db.session.delete(aggregate)
        else:
            aggregate.relevance = 0
            aggregate.read = True
            aggregate.important = False
            aggregate.likes = 0
            aggregate.dislikes = 0
            for news_item in aggregate.news_items:
                aggregate.relevance += news_item.relevance
                aggregate.likes += news_item.likes
                aggregate.dislikes += news_item.dislikes

                if news_item.important is True:
                    aggregate.important = True

                if news_item.read is False:
                    aggregate.read = False

    @classmethod
    def get_news_items_aggregate(cls, source_group, limit):
        if limit is not None:
            limit = datetime.strptime(limit["limit"], "%d.%m.%Y - %H:%M")
        else:
            limit = datetime.now() - timedelta(weeks=8)

        # TODO: Change condition in query to >
        news_item_aggregates = cls.query.filter(cls.osint_source_group_id == source_group).filter(cls.created > limit).all()
        news_item_aggregate_schema = NewsItemAggregateSchema(many=True)
        return news_item_aggregate_schema.dumps(news_item_aggregates)

    @classmethod
    def get_filter_date(cls, limit: str) -> datetime:
        if not limit:
            return datetime.now() - timedelta(weeks=8)
        try:
            return dateparser.parse(limit)
        except Exception:
            return datetime.now() - timedelta(weeks=8)

    @classmethod
    def get_default_news_items_aggregate(cls, limit: str):
        filter_date = cls.get_filter_date(limit)
        source_group = OSINTSourceGroup.get_default()
        news_item_aggregates = cls.query.filter(cls.osint_source_group_id == source_group.id).filter(cls.created > filter_date).all()
        news_item_aggregate_schema = NewsItemAggregateSchema(many=True)
        return news_item_aggregate_schema.dumps(news_item_aggregates)


class NewsItemAggregateSearchIndex(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    data = db.Column(db.String)
    news_item_aggregate_id = db.Column(db.Integer, db.ForeignKey("news_item_aggregate.id"))

    @classmethod
    def remove(cls, aggregate):
        search_index = cls.query.filter_by(news_item_aggregate_id=aggregate.id).first()
        if search_index is not None:
            db.session.delete(search_index)
            db.session.commit()

    @classmethod
    def prepare(cls, aggregate):
        search_index = cls.query.filter_by(news_item_aggregate_id=aggregate.id).first()
        if search_index is None:
            search_index = NewsItemAggregateSearchIndex()
            search_index.news_item_aggregate_id = aggregate.id
            db.session.add(search_index)

        data = aggregate.title
        data += f" {aggregate.description}"
        data += f" {aggregate.comments}"

        for news_item in aggregate.news_items:
            data += " " + news_item.news_item_data.title
            data += " " + news_item.news_item_data.review
            data += " " + news_item.news_item_data.content
            data += " " + news_item.news_item_data.author
            data += " " + news_item.news_item_data.link

            if news_item.news_item_data.tags is not None:
                for tag in news_item.news_item_data.tags:
                    data += f" {tag}"

            for attribute in news_item.news_item_data.attributes:
                data += f" {attribute.value}"

        search_index.data = data.lower()
        db.session.commit()


class NewsItemAttribute(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    key = db.Column(db.String(), nullable=False)
    value = db.Column(db.String(), nullable=False)
    binary_mime_type = db.Column(db.String())
    binary_data = orm.deferred(db.Column(db.LargeBinary))
    created = db.Column(db.DateTime, default=datetime.now)

    remote_node_id = db.Column(db.Integer, db.ForeignKey("remote_node.id"), nullable=True)
    remote_user = db.Column(db.String())

    def __init__(self, key, value, binary_mime_type, binary_value):
        # self.id = id
        self.id = None
        self.key = key
        self.value = value
        self.binary_mime_type = binary_mime_type

        if binary_value:
            self.binary_data = base64.b64decode(binary_value)

    @classmethod
    def find(cls, attribute_id):
        return cls.query.get(attribute_id)


class NewsItemDataNewsItemAttribute(db.Model):
    news_item_data_id = db.Column(db.String, db.ForeignKey("news_item_data.id"), primary_key=True)
    news_item_attribute_id = db.Column(db.Integer, db.ForeignKey("news_item_attribute.id"), primary_key=True)

    @classmethod
    def find(cls, attribute_id):
        news_item_attribute = cls.query.filter(NewsItemDataNewsItemAttribute.news_item_attribute_id == attribute_id).scalar()
        return news_item_attribute


class NewsItemAggregateNewsItemAttribute(db.Model):
    news_item_aggregate_id = db.Column(db.Integer, db.ForeignKey("news_item_aggregate.id"), primary_key=True)
    news_item_attribute_id = db.Column(db.Integer, db.ForeignKey("news_item_attribute.id"), primary_key=True)


class ReportItemNewsItemAggregate(db.Model):
    report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"), primary_key=True)
    news_item_aggregate_id = db.Column(db.Integer, db.ForeignKey("news_item_aggregate.id"), primary_key=True)

    @classmethod
    def assigned(cls, aggregate_id):
        return db.session.query(db.exists().where(ReportItemNewsItemAggregate.news_item_aggregate_id == aggregate_id)).scalar()

    @classmethod
    def count(cls, aggregate_id):
        return cls.query.filter_by(news_item_aggregate_id=aggregate_id).count()



import requests


class RemoteApi:
    def __init__(self, api_url, access_key):
        self.api_url = api_url
        if self.api_url.endswith("/"):
            self.api_url = self.api_url[:-1]
        self.access_key = access_key
        self.headers = {"Authorization": f"Bearer {self.access_key}"}

    def connect(self):
        try:
            response = requests.get(f"{self.api_url}/api/v1/remote/connect", headers=self.headers)

            return response.json(), response.status_code
        except requests.exceptions.ConnectionError:
            return {}, 503

    def disconnect(self):
        try:
            requests.get(f"{self.api_url}/api/v1/remote/disconnect", headers=self.headers)
        except requests.exceptions.ConnectionError:
            return {}, 503

    def get_news_items(self):
        try:
            response = requests.get(f"{self.api_url}/api/v1/remote/sync-news-items", headers=self.headers)

            return response.json(), response.status_code
        except requests.exceptions.ConnectionError:
            return {}, 503

    def confirm_news_items_sync(self, data):
        try:
            response = requests.put(
                f"{self.api_url}/api/v1/remote/sync-news-items",
                headers=self.headers,
                json=data,
            )

            return response.status_code
        except requests.exceptions.ConnectionError:
            return {}, 503

    def get_report_items(self):
        try:
            response = requests.get(f"{self.api_url}/api/v1/remote/sync-report-items", headers=self.headers)

            return response.json(), response.status_code
        except requests.exceptions.ConnectionError:
            return {}, 503

    def confirm_report_items_sync(self, data):
        try:
            response = requests.put(
                f"{self.api_url}/api/v1/remote/sync-report-items",
                headers=self.headers,
                json=data,
            )

            return response.status_code
        except requests.exceptions.ConnectionError:
            return {}, 503



from datetime import datetime

from core.managers.db_manager import db


class TokenBlacklist(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    token = db.Column(db.String(), nullable=False)
    created = db.Column(db.DateTime, default=datetime.now)

    def __init__(self, token):
        self.id = None
        self.token = token

    @classmethod
    def add(cls, token):
        db.session.add(TokenBlacklist(token))
        db.session.commit()

    @classmethod
    def invalid(cls, token):
        return db.session.query(db.exists().where(TokenBlacklist.token == token)).scalar()

    @classmethod
    def delete_older(cls, check_time):
        db.session.query(TokenBlacklist).filter(TokenBlacklist.created < check_time).delete()
        db.session.commit()



from datetime import datetime
import sqlalchemy
from marshmallow import fields
from marshmallow import post_load
from sqlalchemy import orm, func, or_, and_
from sqlalchemy.sql.expression import cast

from core.managers.db_manager import db
from core.model.acl_entry import ACLEntry
from core.model.report_item import ReportItem
from shared.schema.acl_entry import ItemType
from shared.schema.product import ProductPresentationSchema, ProductSchemaBase
from shared.schema.report_item import ReportItemIdSchema


class NewProductSchema(ProductSchemaBase):
    report_items = fields.Nested(ReportItemIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return Product(**data)


class Product(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    created = db.Column(db.DateTime, default=datetime.now)

    product_type_id = db.Column(db.Integer, db.ForeignKey("product_type.id"))
    product_type = db.relationship("ProductType")

    user_id = db.Column(db.Integer, db.ForeignKey("user.id"))
    user = db.relationship("User")

    report_items = db.relationship("ReportItem", secondary="product_report_item")

    def __init__(self, id, title, description, product_type_id, report_items):

        self.id = id if id != -1 else None
        self.title = title
        self.description = description
        self.product_type_id = product_type_id
        self.subtitle = ""
        self.tag = ""

        self.report_items = [ReportItem.find(report_item.id) for report_item in report_items]

    @orm.reconstructor
    def reconstruct(self):
        self.subtitle = self.description
        self.tag = "mdi-file-pdf-outline"

    @classmethod
    def count_all(cls):
        return cls.query.count()

    @classmethod
    def find(cls, product_id):
        product = cls.query.get(product_id)
        return product

    @classmethod
    def get_detail_json(cls, product_id):
        product = cls.query.get(product_id)
        products_schema = ProductPresentationSchema()
        return products_schema.dump(product)

    @classmethod
    def get(cls, filter, offset, limit, user):

        query = (
            db.session.query(
                Product,
                func.count().filter(ACLEntry.id > 0).label("acls"),
                func.count().filter(ACLEntry.access is True).label("access"),
                func.count().filter(ACLEntry.modify is True).label("modify"),
            )
            .distinct()
            .group_by(Product.id)
        )

        query = query.outerjoin(
            ACLEntry,
            and_(
                cast(Product.product_type_id, sqlalchemy.String) == ACLEntry.item_id,
                ACLEntry.item_type == ItemType.PRODUCT_TYPE,
            ),
        )
        query = ACLEntry.apply_query(query, user, True, False, False)

        if "search" in filter and filter["search"] != "":
            search_string = "%" + filter["search"].lower() + "%"
            query = query.filter(
                or_(
                    func.lower(Product.title).like(search_string),
                    func.lower(Product.description).like(search_string),
                )
            )

        if "range" in filter and filter["range"] != "ALL":
            date_limit = datetime.now()
            if filter["range"] == "TODAY":
                date_limit = date_limit.replace(hour=0, minute=0, second=0, microsecond=0)

            if filter["range"] == "WEEK":
                date_limit = date_limit.replace(
                    day=date_limit.day - date_limit.weekday(),
                    hour=0,
                    minute=0,
                    second=0,
                    microsecond=0,
                )

            if filter["range"] == "MONTH":
                date_limit = date_limit.replace(day=1, hour=0, minute=0, second=0, microsecond=0)

            query = query.filter(Product.created >= date_limit)

        if "sort" in filter:
            if filter["sort"] == "DATE_DESC":
                query = query.order_by(db.desc(Product.created))

            elif filter["sort"] == "DATE_ASC":
                query = query.order_by(db.asc(Product.created))

        return query.offset(offset).limit(limit).all(), query.count()

    @classmethod
    def get_json(cls, filter, offset, limit, user):
        results, count = cls.get(filter, offset, limit, user)
        products = []
        for result in results:
            product = result.Product
            product.see = True
            product.access = result.access > 0 or result.acls == 0
            product.modify = result.modify > 0 or result.acls == 0
            products.append(product)

            for report_item in product.report_items:
                report_item.see = True
                report_item.access = True
                report_item.modify = False

        products_schema = ProductPresentationSchema(many=True)
        return {"total_count": count, "items": products_schema.dump(products)}

    @classmethod
    def add_product(cls, product_data, user_id):
        product_schema = NewProductSchema()
        product = product_schema.load(product_data)

        product.user_id = user_id
        db.session.add(product)
        db.session.commit()

        return product

    @classmethod
    def update_product(cls, product_id, product_data):
        product_schema = NewProductSchema()
        product = product_schema.load(product_data)

        original_product = Product.find(product_id)
        original_product.title = product.title
        original_product.description = product.description
        original_product.product_type_id = product.product_type_id
        original_product.report_items = []
        original_product.report_items.extend(product.report_items)

        db.session.commit()

    @classmethod
    def delete(cls, id):
        product = cls.query.get(id)
        if product is not None:
            db.session.delete(product)
            db.session.commit()


class ProductReportItem(db.Model):
    product_id = db.Column(db.Integer, db.ForeignKey("product.id"), primary_key=True)
    report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"), primary_key=True)



import uuid
from marshmallow import fields, post_load
from sqlalchemy import orm, func, or_, text

from core.managers.db_manager import db
from core.model.report_item import ReportItem
from core.model.user import User
from core.model.notification_template import NotificationTemplate
from shared.schema.asset import (
    AssetCpeSchema,
    AssetSchema,
    AssetPresentationSchema,
    AssetGroupSchema,
    AssetGroupPresentationSchema,
)
from shared.schema.user import UserIdSchema
from shared.schema.notification_template import NotificationTemplateIdSchema


class NewAssetCpeSchema(AssetCpeSchema):
    @post_load
    def make(self, data, **kwargs):
        return AssetCpe(**data)


class AssetCpe(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    value = db.Column(db.String())

    asset_id = db.Column(db.Integer, db.ForeignKey("asset.id"))
    asset = db.relationship("Asset")

    def __init__(self, value):
        self.id = None
        self.value = value


class NewAssetSchema(AssetSchema):
    asset_cpes = fields.Nested(NewAssetCpeSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return Asset(**data)


class Asset(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    serial = db.Column(db.String())
    description = db.Column(db.String())

    asset_group_id = db.Column(db.String, db.ForeignKey("asset_group.id"))
    asset_group = db.relationship("AssetGroup")

    asset_cpes = db.relationship("AssetCpe", cascade="all, delete-orphan", back_populates="asset")

    vulnerabilities = db.relationship("AssetVulnerability", cascade="all, delete-orphan", back_populates="asset")
    vulnerabilities_count = db.Column(db.Integer, default=0)

    def __init__(self, id, name, serial, description, asset_group_id, asset_cpes):
        self.id = None
        self.name = name
        self.serial = serial
        self.description = description
        self.asset_group_id = asset_group_id
        self.asset_cpes = asset_cpes
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-laptop"

    @classmethod
    def get_by_cpe(cls, cpes):

        if len(cpes) > 0:
            query_string = "SELECT DISTINCT asset_id FROM asset_cpe WHERE value LIKE ANY(:cpes) OR {}"
            params = {"cpes": cpes}

            inner_query = ""
            for i in range(len(cpes)):
                if i > 0:
                    inner_query += " OR "
                param = "cpe" + str(i)
                inner_query += ":" + param + " LIKE value"
                params[param] = cpes[i]

            result = db.engine.execute(text(query_string.format(inner_query)), params)

            return [cls.query.get(row[0]) for row in result]
        else:
            return []

    @classmethod
    def remove_vulnerability(cls, report_item_id):
        vulnerabilities = AssetVulnerability.get_by_report(report_item_id)
        for vulnerability in vulnerabilities:
            vulnerability.asset.vulnerabilities_count -= 1
            db.session.delete(vulnerability)

    def add_vulnerability(self, report_item):
        for vulnerability in self.vulnerabilities:
            if vulnerability.report_item.id == report_item.id:
                return

        vulnerability = AssetVulnerability(self.id, report_item.id)
        db.session.add(vulnerability)
        self.vulnerabilities_count += 1

    def update_vulnerabilities(self):
        cpes = []
        for cpe in self.asset_cpes:
            cpes.append(cpe.value)

        report_item_ids = ReportItem.get_by_cpe(cpes)

        solved = []
        for vulnerability in self.vulnerabilities:
            if vulnerability.solved is True:
                solved.append(vulnerability.report_item_id)

        self.vulnerabilities = []
        self.vulnerabilities_count = 0
        for report_item_id in report_item_ids:
            vulnerability = AssetVulnerability(self.id, report_item_id)
            if report_item_id in solved:
                vulnerability.solved = True
            else:
                self.vulnerabilities_count += 1
            self.vulnerabilities.append(vulnerability)

    @classmethod
    def solve_vulnerability(cls, user, group_id, asset_id, report_item_id, solved):
        asset = cls.query.get(asset_id)
        if AssetGroup.access_allowed(user, asset.asset_group_id):
            for vulnerability in asset.vulnerabilities:
                if vulnerability.report_item_id == report_item_id:
                    if solved is not vulnerability.solved:
                        if solved is True:
                            asset.vulnerabilities_count -= 1
                        else:
                            asset.vulnerabilities_count += 1
                    vulnerability.solved = solved
                    db.session.commit()
                    return

    @classmethod
    def get(cls, group_id, search, sort, vulnerable):
        query = cls.query.filter(Asset.asset_group_id == group_id)

        if vulnerable is not None:
            if vulnerable == "true":
                query = query.filter(Asset.vulnerabilities_count > 0)

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.join(AssetCpe, Asset.id == AssetCpe.asset_id).filter(
                or_(
                    func.lower(Asset.name).like(search_string),
                    func.lower(Asset.description).like(search_string),
                    func.lower(Asset.serial).like(search_string),
                    func.lower(AssetCpe.value).like(search_string),
                )
            )

        if sort is not None:
            if sort == "ALPHABETICAL":
                query = query.order_by(db.asc(Asset.name))
            else:
                query = query.order_by(db.desc(Asset.vulnerabilities_count))

        return query.all(), query.count()

    @classmethod
    def get_all_json(cls, user, group_id, search, sort, vulnerable):
        if AssetGroup.access_allowed(user, group_id):
            assets, count = cls.get(group_id, search, sort, vulnerable)
            asset_schema = AssetPresentationSchema(many=True)
            items = asset_schema.dump(assets)
            return {"total_count": count, "items": items}

    @classmethod
    def add(cls, user, group_id, data):
        schema = NewAssetSchema()
        asset = schema.load(data)
        asset.asset_group_id = group_id
        if AssetGroup.access_allowed(user, group_id):
            db.session.add(asset)
            asset.update_vulnerabilities()
            db.session.commit()

    @classmethod
    def update(cls, user, group_id, asset_id, data):
        asset = cls.query.get(asset_id)
        if AssetGroup.access_allowed(user, asset.asset_group_id):
            schema = NewAssetSchema()
            updated_asset = schema.load(data)
            asset.name = updated_asset.name
            asset.serial = updated_asset.serial
            asset.description = updated_asset.description
            asset.asset_cpes = updated_asset.asset_cpes
            asset.update_vulnerabilities()
            db.session.commit()

    @classmethod
    def delete(cls, user, group_id, id):
        asset = cls.query.get(id)
        if AssetGroup.access_allowed(user, asset.asset_group_id):
            db.session.delete(asset)
            db.session.commit()


class AssetVulnerability(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    solved = db.Column(db.Boolean, default=False)

    asset_id = db.Column(db.Integer, db.ForeignKey("asset.id"))
    asset = db.relationship("Asset", back_populates="vulnerabilities")

    report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"))
    report_item = db.relationship("ReportItem")

    def __init__(self, asset_id, report_item_id):
        self.id = None
        self.asset_id = asset_id
        self.report_item_id = report_item_id

    @classmethod
    def get_by_report(cls, report_id):
        return cls.query.filter_by(report_item_id=report_id).all()


class NewAssetGroupGroupSchema(AssetGroupSchema):
    users = fields.Nested(UserIdSchema, many=True)
    templates = fields.Nested(NotificationTemplateIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return AssetGroup(**data)


class AssetGroup(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    templates = db.relationship("NotificationTemplate", secondary="asset_group_notification_template")

    organizations = db.relationship("Organization", secondary="asset_group_organization")
    users = db.relationship("User", secondary="asset_group_user")

    def __init__(self, id, name, description, users, templates):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.organizations = []
        self.users = []
        for user in users:
            self.users.append(User.find_by_id(user.id))

        self.templates = []
        for template in templates:
            self.templates.append(NotificationTemplate.find(template.id))

        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-folder-multiple"

    @classmethod
    def find(cls, group_id):
        group = cls.query.get(group_id)
        return group

    @classmethod
    def access_allowed(cls, user, group_id):
        group = cls.query.get(group_id)
        return any(org in user.organizations for org in group.organizations)

    @classmethod
    def get(cls, search, organization):
        query = cls.query

        if organization is not None:
            query = query.join(
                AssetGroupOrganization,
                AssetGroup.id == AssetGroupOrganization.asset_group_id,
            )

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(AssetGroup.name).like(search_string),
                    func.lower(AssetGroup.description).like(search_string),
                )
            )

        return query.order_by(db.asc(AssetGroup.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, user, search):
        groups, count = cls.get(search, user.organizations[0])
        permissions = user.get_permissions()
        if "MY_ASSETS_CONFIG" not in permissions:
            for group in groups[:]:
                if len(group.users) > 0:
                    found = False
                    for accessed_user in group.users:
                        if accessed_user.id == user.id:
                            found = True
                            break

                    if found is False:
                        groups.remove(group)
                        count -= 1

        group_schema = AssetGroupPresentationSchema(many=True)
        return {"total_count": count, "items": group_schema.dump(groups)}

    @classmethod
    def add(cls, user, data):
        new_group_schema = NewAssetGroupGroupSchema()
        group = new_group_schema.load(data)
        group.organizations = user.organizations
        for added_user in group.users[:]:
            if not any(org in added_user.organizations for org in group.organizations):
                group.users.remove(added_user)

        for added_template in group.templates[:]:
            if not any(org in added_template.organizations for org in group.organizations):
                group.temlates.remove(added_template)

        db.session.add(group)
        db.session.commit()

    @classmethod
    def delete(cls, user, group_id):
        group = cls.query.get(group_id)
        if any(org in user.organizations for org in group.organizations):
            db.session.delete(group)
            db.session.commit()

    @classmethod
    def update(cls, user, group_id, data):
        new_group_schema = NewAssetGroupGroupSchema()
        updated_group = new_group_schema.load(data)
        group = cls.query.get(group_id)
        if any(org in user.organizations for org in group.organizations):
            group.name = updated_group.name
            group.description = updated_group.description
            group.users = []
            for added_user in updated_group.users:
                if any(org in added_user.organizations for org in group.organizations):
                    group.users.append(added_user)

            group.templates = []
            for added_template in updated_group.templates:
                if any(org in added_template.organizations for org in group.organizations):
                    group.templates.append(added_template)

            db.session.commit()


class AssetGroupOrganization(db.Model):
    asset_group_id = db.Column(db.String, db.ForeignKey("asset_group.id"), primary_key=True)
    organization_id = db.Column(db.Integer, db.ForeignKey("organization.id"), primary_key=True)


class AssetGroupUser(db.Model):
    asset_group_id = db.Column(db.String, db.ForeignKey("asset_group.id"), primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), primary_key=True)


class AssetGroupNotificationTemplate(db.Model):
    asset_group_id = db.Column(db.String, db.ForeignKey("asset_group.id"), primary_key=True)
    notification_template_id = db.Column(db.Integer, db.ForeignKey("notification_template.id"), primary_key=True)



from marshmallow import fields, post_load
import uuid

from core.managers.db_manager import db
from core.model.parameter import NewParameterSchema
from shared.schema.presenter import PresenterSchema


class NewPresenterSchema(PresenterSchema):
    parameters = fields.List(fields.Nested(NewParameterSchema))

    @post_load
    def make(self, data, **kwargs):
        return Presenter(**data)


class Presenter(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    type = db.Column(db.String(), nullable=False)

    parameters = db.relationship("Parameter", secondary="presenter_parameter")

    node_id = db.Column(db.String, db.ForeignKey("presenters_node.id"))
    node = db.relationship("PresentersNode", back_populates="presenters")

    product_types = db.relationship("ProductType", back_populates="presenter")

    def __init__(self, name, description, type, parameters):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.type = type
        self.parameters = parameters

    @classmethod
    def create_all(cls, presenters_data):
        new_presenter_schema = NewPresenterSchema(many=True)
        return new_presenter_schema.load(presenters_data)


class PresenterParameter(db.Model):
    presenter_id = db.Column(db.String, db.ForeignKey("presenter.id"), primary_key=True)
    parameter_id = db.Column(db.Integer, db.ForeignKey("parameter.id"), primary_key=True)



from core.model.bots_node import BotsNode
from core.model.bot import Bot
from core.model.bot_preset import BotPreset
from core.remote.bots_api import BotsApi
from shared.schema.bots_node import BotsNode as BotsNodeSchema
from core.managers.log_manager import logger


def get_bots_info(node: BotsNodeSchema):
    try:
        bots_info, status_code = BotsApi(node.api_url, node.api_key).get_bots_info()
    except ConnectionError:
        return f"Connection error: Could not reach {node.api_url}", 500
    except Exception:
        logger.log_debug_trace(f"Couldn't add Bot Node: {node.name}")
        return f"Couldn't add Bot node: {node.name}", 500

    if status_code != 200:
        return None, status_code

    return Bot.create_all(bots_info), status_code


def add_bots_node(data):
    try:
        logger.log_info(data)
        bots_info = data.pop("bots_info")
        node = BotsNodeSchema.create(data)
    except Exception as e:
        logger.log_debug_trace()
        return str(e), 500

    try:
        bots = Bot.create_all(bots_info)
        BotsNode.add_new(data, bots)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Bot Node: {node.name}")
        return f"Couldn't add Bot Node: {node.name}", 500

    return node.id, 200


def update_bots_node(node_id, data):
    node = BotsNodeSchema.create(data)
    bots, status_code = get_bots_info(node)

    if status_code != 200:
        return bots, status_code

    try:
        BotsNode.update(node_id, data, bots)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Bot Node: {node.name}")
        return f"Couldn't add Bot node: {node.name}", 500

    return node.id, status_code


def verify_api_key(api_key):
    return BotsNode.exists_by_api_key(api_key)


def add_bot_preset(data):
    BotPreset.add_new(data)



import io

from flask import request, send_file
from flask_restful import Resource

from core.managers import (
    auth_manager,
    sse_manager,
    remote_manager,
    presenters_manager,
    publishers_manager,
    bots_manager,
    external_auth_manager,
    collectors_manager,
)
from core.managers.log_manager import logger
from core.managers.auth_manager import auth_required, get_user_from_jwt
from core.model import (
    acl_entry,
    remote,
    presenters_node,
    publisher_preset,
    publishers_node,
    bots_node,
    bot_preset,
    attribute,
    collectors_node,
    organization,
    osint_source,
    product_type,
    report_item_type,
    role,
    user,
    word_list,
)
from core.model.news_item import NewsItemAggregate
from core.model.permission import Permission
from shared.schema.role import PermissionSchema


class DictionariesReload(Resource):
    @auth_required("CONFIG_ATTRIBUTE_UPDATE")
    def get(self, dictionary_type):
        attribute.Attribute.load_dictionaries(dictionary_type)
        return "success", 200


class Attributes(Resource):
    @auth_required("CONFIG_ATTRIBUTE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return attribute.Attribute.get_all_json(search)

    @auth_required("CONFIG_ATTRIBUTE_CREATE")
    def post(self):
        attribute.Attribute.add_attribute(request.json)


class Attribute(Resource):
    @auth_required("CONFIG_ATTRIBUTE_UPDATE")
    def put(self, attribute_id):
        attribute.Attribute.update(attribute_id, request.json)

    @auth_required("CONFIG_ATTRIBUTE_DELETE")
    def delete(self, attribute_id):
        return attribute.Attribute.delete_attribute(attribute_id)


class AttributeEnums(Resource):
    @auth_required("CONFIG_ATTRIBUTE_ACCESS")
    def get(self, attribute_id):
        search = request.args.get(key="search", default=None)
        offset = request.args.get(key="offset", default=0)
        limit = request.args.get(key="limit", default=10)
        return attribute.AttributeEnum.get_for_attribute_json(attribute_id, search, offset, limit)

    @auth_required("CONFIG_ATTRIBUTE_UPDATE")
    def post(self, attribute_id):
        attribute.AttributeEnum.add(attribute_id, request.json)


class AttributeEnum(Resource):
    @auth_required("CONFIG_ATTRIBUTE_UPDATE")
    def put(self, attribute_id, enum_id):
        attribute.AttributeEnum.update(enum_id, request.json)

    @auth_required("CONFIG_ATTRIBUTE_UPDATE")
    def delete(self, attribute_id, enum_id):
        return attribute.AttributeEnum.delete(enum_id)


class ReportItemTypesConfig(Resource):
    @auth_required("CONFIG_REPORT_TYPE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return report_item_type.ReportItemType.get_all_json(search, auth_manager.get_user_from_jwt(), False)

    @auth_required("CONFIG_REPORT_TYPE_CREATE")
    def post(self):
        report_item_type.ReportItemType.add_report_item_type(request.json)


class ReportItemType(Resource):
    @auth_required("CONFIG_REPORT_TYPE_UPDATE")
    def put(self, type_id):
        report_item_type.ReportItemType.update(type_id, request.json)

    @auth_required("CONFIG_REPORT_TYPE_DELETE")
    def delete(self, type_id):
        return report_item_type.ReportItemType.delete_report_item_type(type_id)


class ProductTypes(Resource):
    @auth_required("CONFIG_PRODUCT_TYPE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return product_type.ProductType.get_all_json(search, auth_manager.get_user_from_jwt(), False)

    @auth_required("CONFIG_PRODUCT_TYPE_CREATE")
    def post(self):
        product_type.ProductType.add_new(request.json)


class ProductType(Resource):
    @auth_required("CONFIG_PRODUCT_TYPE_UPDATE")
    def put(self, type_id):
        product_type.ProductType.update(type_id, request.json)

    @auth_required("CONFIG_PRODUCT_TYPE_DELETE")
    def delete(self, type_id):
        return product_type.ProductType.delete(type_id)


class Permissions(Resource):
    @auth_required("CONFIG_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return Permission.get_all_json(search)


class ExternalPermissions(Resource):
    @auth_required("MY_ASSETS_CONFIG")
    def get(self):
        permissions = auth_manager.get_external_permissions()
        permissions_schema = PermissionSchema(many=True)
        return {
            "total_count": len(permissions),
            "items": permissions_schema.dump(permissions),
        }


class Roles(Resource):
    @auth_required("CONFIG_ROLE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return role.Role.get_all_json(search)

    @auth_required("CONFIG_ROLE_CREATE")
    def post(self):
        role.Role.add_new(request.json)


class Role(Resource):
    @auth_required("CONFIG_ROLE_UPDATE")
    def put(self, role_id):
        role.Role.update(role_id, request.json)

    @auth_required("CONFIG_ROLE_DELETE")
    def delete(self, role_id):
        return role.Role.delete(role_id)


class ACLEntries(Resource):
    @auth_required("CONFIG_ACL_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return acl_entry.ACLEntry.get_all_json(search)

    @auth_required("CONFIG_ACL_CREATE")
    def post(self):
        acl_entry.ACLEntry.add_new(request.json)


class ACLEntry(Resource):
    @auth_required("CONFIG_ACL_UPDATE")
    def put(self, acl_id):
        acl_entry.ACLEntry.update(acl_id, request.json)

    @auth_required("CONFIG_ACL_DELETE")
    def delete(self, acl_id):
        return acl_entry.ACLEntry.delete(acl_id)


class Organizations(Resource):
    @auth_required("CONFIG_ORGANIZATION_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return organization.Organization.get_all_json(search)

    @auth_required("CONFIG_ORGANIZATION_CREATE")
    def post(self):
        organization.Organization.add_new(request.json)


class Organization(Resource):
    @auth_required("CONFIG_ORGANIZATION_UPDATE")
    def put(self, organization_id):
        organization.Organization.update(organization_id, request.json)

    @auth_required("CONFIG_ORGANIZATION_DELETE")
    def delete(self, organization_id):
        return organization.Organization.delete(organization_id)


class Users(Resource):
    @auth_required("CONFIG_USER_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return user.User.get_all_json(search)

    @auth_required("CONFIG_USER_CREATE")
    def post(self):
        try:
            external_auth_manager.create_user(request.json)
        except Exception as ex:
            logger.log_debug(ex)
            logger.store_data_error_activity(get_user_from_jwt(), "Could not create user in external auth system")
            return "", 400

        user.User.add_new(request.json)


class User(Resource):
    @auth_required("CONFIG_USER_UPDATE")
    def put(self, user_id):
        original_user = user.User.find_by_id(user_id)
        original_username = original_user.username

        try:
            external_auth_manager.update_user(request.json, original_username)
        except Exception as ex:
            logger.log_debug(ex)
            logger.store_data_error_activity(get_user_from_jwt(), "Could not update user in external auth system")
            return "", 400

        user.User.update(user_id, request.json)

    @auth_required("CONFIG_USER_DELETE")
    def delete(self, user_id):
        original_user = user.User.find_by_id(user_id)
        original_username = original_user.username

        user.User.delete(user_id)

        try:
            external_auth_manager.delete_user(original_username)
        except Exception as ex:
            logger.log_debug(ex)
            logger.store_data_error_activity(get_user_from_jwt(), "Could not delete user in external auth system")
            return "", 400


class ExternalUsers(Resource):
    @auth_required("CONFIG_USER_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return user.User.get_all_external_json(auth_manager.get_user_from_jwt(), search)

    @auth_required("MY_ASSETS_CONFIG")
    def post(self):
        permissions = auth_manager.get_external_permissions_ids()
        user.User.add_new_external(auth_manager.get_user_from_jwt(), permissions, request.json)


class ExternalUser(Resource):
    @auth_required("CONFIG_USER_ACCESS")
    def put(self, user_id):
        permissions = auth_manager.get_external_permissions_ids()
        user.User.update_external(auth_manager.get_user_from_jwt(), permissions, user_id, request.json)

    @auth_required("MY_ASSETS_CONFIG")
    def delete(self, user_id):
        return user.User.delete_external(auth_manager.get_user_from_jwt(), user_id)


class WordLists(Resource):
    @auth_required("CONFIG_WORD_LIST_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return word_list.WordList.get_all_json(search, auth_manager.get_user_from_jwt(), False)

    @auth_required("CONFIG_WORD_LIST_CREATE")
    def post(self):
        word_list.WordList.add_new(request.json)


class WordList(Resource):
    @auth_required("CONFIG_WORD_LIST_DELETE")
    def delete(self, word_list_id):
        return word_list.WordList.delete(word_list_id)

    @auth_required("CONFIG_WORD_LIST_UPDATE")
    def put(self, word_list_id):
        word_list.WordList.update(word_list_id, request.json)


class CollectorsNodes(Resource):
    @auth_required("CONFIG_COLLECTORS_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return collectors_node.CollectorsNode.get_all_json(search)

    @auth_required("CONFIG_COLLECTORS_NODE_CREATE")
    def post(self):
        return collectors_manager.add_collectors_node(request.json)

    @auth_required("CONFIG_COLLECTORS_NODE_UPDATE")
    def put(self, node_id):
        collectors_manager.update_collectors_node(node_id, request.json)

    @auth_required("CONFIG_COLLECTORS_NODE_DELETE")
    def delete(self, node_id):
        collectors_node.CollectorsNode.delete(node_id)


class OSINTSources(Resource):
    @auth_required("CONFIG_OSINT_SOURCE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return osint_source.OSINTSource.get_all_json(search)

    @auth_required("CONFIG_OSINT_SOURCE_CREATE")
    def post(self):
        collectors_manager.add_osint_source(request.json)


class OSINTSource(Resource):
    @auth_required("CONFIG_OSINT_SOURCE_UPDATE")
    def put(self, source_id):
        updated_osint_source, default_group = collectors_manager.update_osint_source(source_id, request.json)
        if default_group is not None:
            NewsItemAggregate.reassign_to_new_groups(updated_osint_source.id, default_group.id)

    @auth_required("CONFIG_OSINT_SOURCE_DELETE")
    def delete(self, source_id):
        collectors_manager.delete_osint_source(source_id)


class OSINTSourcesExport(Resource):
    @auth_required("CONFIG_OSINT_SOURCE_ACCESS")
    def post(self):
        data = collectors_manager.export_osint_sources(request.json)
        return send_file(
            io.BytesIO(data),
            download_name="osint_sources_export.json",
            mimetype="application/json",
            as_attachment=True,
        )


class OSINTSourcesImport(Resource):
    @auth_required("CONFIG_OSINT_SOURCE_CREATE")
    def post(self):
        if file := request.files.get("file"):
            collectors_node_id = request.form["collectors_node_id"]
            collectors_manager.import_osint_sources(collectors_node_id, file)


class OSINTSourceGroups(Resource):
    @auth_required("CONFIG_OSINT_SOURCE_GROUP_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return osint_source.OSINTSourceGroup.get_all_json(search, auth_manager.get_user_from_jwt(), False)

    @auth_required("CONFIG_OSINT_SOURCE_GROUP_CREATE")
    def post(self):
        osint_source.OSINTSourceGroup.add(request.json)


class OSINTSourceGroup(Resource):
    @auth_required("CONFIG_OSINT_SOURCE_GROUP_UPDATE")
    def put(self, group_id):
        sources_in_default_group, message, code = osint_source.OSINTSourceGroup.update(group_id, request.json)
        if sources_in_default_group is not None:
            default_group = osint_source.OSINTSourceGroup.get_default()
            for source in sources_in_default_group:
                NewsItemAggregate.reassign_to_new_groups(source.id, default_group.id)

        return message, code

    @auth_required("CONFIG_OSINT_SOURCE_GROUP_DELETE")
    def delete(self, group_id):
        return osint_source.OSINTSourceGroup.delete(group_id)


class RemoteAccesses(Resource):
    @auth_required("CONFIG_REMOTE_ACCESS_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return remote.RemoteAccess.get_all_json(search)

    @auth_required("CONFIG_REMOTE_ACCESS_CREATE")
    def post(self):
        remote.RemoteAccess.add(request.json)


class RemoteAccess(Resource):
    @auth_required("CONFIG_REMOTE_ACCESS_UPDATE")
    def put(self, remote_access_id):
        event_id, disconnect = remote.RemoteAccess.update(remote_access_id, request.json)
        if disconnect:
            sse_manager.remote_access_disconnect([event_id])

    @auth_required("CONFIG_REMOTE_ACCESS_DELETE")
    def delete(self, remote_access_id):
        return remote.RemoteAccess.delete(remote_access_id)


class RemoteNodes(Resource):
    @auth_required("CONFIG_REMOTE_ACCESS_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return remote.RemoteNode.get_all_json(search)

    @auth_required("CONFIG_REMOTE_ACCESS_CREATE")
    def post(self):
        remote.RemoteNode.add(request.json)


class RemoteNode(Resource):
    @auth_required("CONFIG_REMOTE_ACCESS_UPDATE")
    def put(self, remote_node_id):
        if remote.RemoteNode.update(id, request.json) is False:
            remote_manager.disconnect_from_node(remote_node_id)

    @auth_required("CONFIG_REMOTE_ACCESS_DELETE")
    def delete(self, remote_node_id):
        remote_manager.disconnect_from_node(remote_node_id)
        return remote.RemoteNode.delete(id)


class RemoteNodeConnect(Resource):
    @auth_required("CONFIG_REMOTE_ACCESS_ACCESS")
    def get(self, remote_node_id):
        return remote_manager.connect_to_node(remote_node_id)


class PresentersNodes(Resource):
    @auth_required("CONFIG_PRESENTERS_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return presenters_node.PresentersNode.get_all_json(search)

    @auth_required("CONFIG_PRESENTERS_NODE_CREATE")
    def post(self):
        return "", presenters_manager.add_presenters_node(request.json)

    @auth_required("CONFIG_PRESENTERS_NODE_UPDATE")
    def put(self, node_id):
        presenters_manager.update_presenters_node(node_id, request.json)

    @auth_required("CONFIG_PRESENTERS_NODE_DELETE")
    def delete(self, node_id):
        return presenters_node.PresentersNode.delete(node_id)


class PublisherNodes(Resource):
    @auth_required("CONFIG_PUBLISHERS_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return publishers_node.PublishersNode.get_all_json(search)

    @auth_required("CONFIG_PUBLISHERS_NODE_CREATE")
    def post(self):
        return "", publishers_manager.add_publishers_node(request.json)

    @auth_required("CONFIG_PUBLISHERS_NODE_UPDATE")
    def put(self, node_id):
        publishers_manager.update_publishers_node(node_id, request.json)

    @auth_required("CONFIG_PUBLISHERS_NODE_DELETE")
    def delete(self, node_id):
        return publishers_node.PublishersNode.delete(node_id)


class PublisherPresets(Resource):
    @auth_required("CONFIG_PUBLISHER_PRESET_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return publisher_preset.PublisherPreset.get_all_json(search)

    @auth_required("CONFIG_PUBLISHER_PRESET_CREATE")
    def post(self):
        publishers_manager.add_publisher_preset(request.json)


class PublisherPreset(Resource):
    @auth_required("CONFIG_PUBLISHER_PRESET_UPDATE")
    def put(self, preset_id):
        publisher_preset.PublisherPreset.update(preset_id, request.json)

    @auth_required("CONFIG_PUBLISHER_PRESET_DELETE")
    def delete(self, preset_id):
        return publisher_preset.PublisherPreset.delete(preset_id)


class Nodes(Resource):
    @auth_required("CONFIG_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        publishers_nodes = publishers_node.PublishersNode.get_all_json(search)
        bots_nodes = bots_node.BotsNode.get_all_json(search)
        collectors_nodes = collectors_node.CollectorsNode.get_all_json(search)
        presenters_nodes = presenters_node.PresentersNode.get_all_json(search)
        total_count = (
            publishers_nodes["total_count"] + bots_nodes["total_count"] + collectors_nodes["total_count"] + presenters_nodes["total_count"]
        )
        items = publishers_nodes["items"] + bots_nodes["items"] + collectors_nodes["items"] + presenters_nodes["items"]
        return {"total_count": total_count, "items": items}


class BotNodes(Resource):
    @auth_required("CONFIG_BOTS_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return bots_node.BotsNode.get_all_json(search)

    @auth_required("CONFIG_BOTS_NODE_CREATE")
    def post(self):
        return bots_manager.add_bots_node(request.json)

    @auth_required("CONFIG_BOTS_NODE_UPDATE")
    def put(self, node_id):
        bots_manager.update_bots_node(node_id, request.json)

    @auth_required("CONFIG_BOTS_NODE_DELETE")
    def delete(self, node_id):
        return bots_node.BotsNode.delete(node_id)


class BotPresets(Resource):
    @auth_required("CONFIG_BOT_PRESET_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return bot_preset.BotPreset.get_all_json(search)

    @auth_required("CONFIG_BOT_PRESET_CREATE")
    def post(self):
        bots_manager.add_bot_preset(request.json)


class BotPreset(Resource):
    @auth_required("CONFIG_BOT_PRESET_UPDATE")
    def put(self, preset_id):
        bot_preset.BotPreset.update(preset_id, request.json)

    @auth_required("CONFIG_BOT_PRESET_DELETE")
    def delete(self, preset_id):
        return bot_preset.BotPreset.delete(preset_id)


def initialize(api):
    api.add_resource(
        DictionariesReload,
        "/api/v1/config/reload-enum-dictionaries/<string:dictionary_type>",
    )
    api.add_resource(Attributes, "/api/v1/config/attributes")
    api.add_resource(Attribute, "/api/v1/config/attributes/<int:attribute_id>")
    api.add_resource(AttributeEnums, "/api/v1/config/attributes/<int:attribute_id>/enums")
    api.add_resource(
        AttributeEnum,
        "/api/v1/config/attributes/<int:attribute_id>/enums/<int:enum_id>",
    )

    api.add_resource(ReportItemTypesConfig, "/api/v1/config/report-item-types")
    api.add_resource(ReportItemType, "/api/v1/config/report-item-types/<int:type_id>")

    api.add_resource(ProductTypes, "/api/v1/config/product-types")
    api.add_resource(ProductType, "/api/v1/config/product-types/<int:type_id>")

    api.add_resource(Permissions, "/api/v1/config/permissions")
    api.add_resource(ExternalPermissions, "/api/v1/config/external-permissions")
    api.add_resource(Roles, "/api/v1/config/roles")
    api.add_resource(Role, "/api/v1/config/roles/<int:role_id>")
    api.add_resource(ACLEntries, "/api/v1/config/acls")
    api.add_resource(ACLEntry, "/api/v1/config/acls/<int:acl_id>")

    api.add_resource(Organizations, "/api/v1/config/organizations")
    api.add_resource(Organization, "/api/v1/config/organizations/<int:organization_id>")

    api.add_resource(Users, "/api/v1/config/users")
    api.add_resource(User, "/api/v1/config/users/<int:user_id>")

    api.add_resource(ExternalUsers, "/api/v1/config/external-users")
    api.add_resource(ExternalUser, "/api/v1/config/external-users/<int:user_id>")

    api.add_resource(WordLists, "/api/v1/config/word-lists")
    api.add_resource(WordList, "/api/v1/config/word-lists/<int:word_list_id>")

    api.add_resource(CollectorsNodes, "/api/v1/config/collectors-nodes", "/api/v1/config/collectors-nodes/<string:node_id>")
    api.add_resource(OSINTSources, "/api/v1/config/osint-sources")
    api.add_resource(OSINTSource, "/api/v1/config/osint-sources/<string:source_id>")
    api.add_resource(OSINTSourcesExport, "/api/v1/config/export-osint-sources")
    api.add_resource(OSINTSourcesImport, "/api/v1/config/import-osint-sources")
    api.add_resource(OSINTSourceGroups, "/api/v1/config/osint-source-groups")
    api.add_resource(OSINTSourceGroup, "/api/v1/config/osint-source-groups/<string:group_id>")

    api.add_resource(RemoteAccesses, "/api/v1/config/remote-accesses")
    api.add_resource(RemoteAccess, "/api/v1/config/remote-accesses/<int:remote_access_id>")

    api.add_resource(RemoteNodes, "/api/v1/config/remote-nodes")
    api.add_resource(RemoteNode, "/api/v1/config/remote-nodes/<int:remote_node_id>")
    api.add_resource(RemoteNodeConnect, "/api/v1/config/remote-nodes/<int:remote_node_id>/connect")

    api.add_resource(PresentersNodes, "/api/v1/config/presenters-nodes", "/api/v1/config/presenters-nodes/<string:node_id>")
    api.add_resource(PublisherNodes, "/api/v1/config/publishers-nodes", "/api/v1/config/publishers-nodes/<string:node_id>")
    api.add_resource(PublisherPresets, "/api/v1/config/publishers-presets")
    api.add_resource(PublisherPreset, "/api/v1/config/publishers-presets/<string:preset_id>")

    api.add_resource(BotNodes, "/api/v1/config/bots-nodes", "/api/v1/config/bots-nodes/<string:node_id>")
    api.add_resource(BotPresets, "/api/v1/config/bots-presets")
    api.add_resource(BotPreset, "/api/v1/config/bots-presets/<string:preset_id>")

    api.add_resource(Nodes, "/api/v1/config/nodes", "/api/v1/config/nodes/<string:node_id>")



import base64
from flask import Response
from flask import request
from flask_restful import Resource

from core.managers import auth_manager, presenters_manager, publishers_manager
from core.managers.log_manager import logger
from core.managers.auth_manager import auth_required, ACLCheck
from core.model import product, product_type, publisher_preset


class Products(Resource):
    @auth_required("PUBLISH_ACCESS")
    def get(self):
        try:
            filter = {}
            if "search" in request.args and request.args["search"]:
                filter["search"] = request.args["search"]
            if "range" in request.args and request.args["range"]:
                filter["range"] = request.args["range"]
            if "sort" in request.args and request.args["sort"]:
                filter["sort"] = request.args["sort"]

            offset = None
            if "offset" in request.args and request.args["offset"]:
                offset = int(request.args["offset"])

            limit = 50
            if "limit" in request.args and request.args["limit"]:
                limit = min(int(request.args["limit"]), 200)
        except Exception as ex:
            logger.log_debug(ex)
            return "", 400

        return product.Product.get_json(filter, offset, limit, auth_manager.get_user_from_jwt())

    @auth_required("PUBLISH_CREATE")
    def post(self):
        user = auth_manager.get_user_from_jwt()
        new_product = product.Product.add_product(request.json, user.id)
        return new_product.id


class Product(Resource):
    @auth_required("PUBLISH_UPDATE", ACLCheck.PRODUCT_TYPE_ACCESS)
    def get(self, product_id):
        return product.Product.get_detail_json(product_id)

    @auth_required("PUBLISH_UPDATE", ACLCheck.PRODUCT_TYPE_MODIFY)
    def put(self, product_id):
        product.Product.update_product(product_id, request.json)

    @auth_required("PUBLISH_DELETE", ACLCheck.PRODUCT_TYPE_MODIFY)
    def delete(self, product_id):
        return product.Product.delete(product_id)


class PublishProduct(Resource):
    @auth_required("PUBLISH_PRODUCT")
    def post(self, product_id, publisher_id):
        product_data, status_code = presenters_manager.generate_product(product_id)
        if status_code == 200:
            return publishers_manager.publish(
                publisher_preset.PublisherPreset.find(publisher_id),
                product_data,
                None,
                None,
                None,
            )
        else:
            return "Failed to generate product", status_code


class ProductsOverview(Resource):
    def get(self, product_id):
        if "jwt" in request.args:
            user = auth_manager.decode_user_from_jwt(request.args["jwt"])
            if user is not None:
                permissions = user.get_permissions()
                if "PUBLISH_ACCESS" in permissions:
                    if product_type.ProductType.allowed_with_acl(product_id, user, False, True, False):
                        product_data, status_code = presenters_manager.generate_product(product_id)
                        if status_code == 200:
                            return Response(
                                base64.b64decode(product_data["data"]),
                                mimetype=product_data["mime_type"],
                            )
                        else:
                            return "Failed to generate product", status_code
                    else:
                        logger.store_auth_error_activity("Unauthorized access attempt to Product Type")
                else:
                    logger.store_auth_error_activity("Insufficient permissions")
            else:
                logger.store_auth_error_activity("Invalid JWT")
        else:
            logger.store_auth_error_activity("Missing JWT")


def initialize(api):
    api.add_resource(Products, "/api/v1/publish/products")
    api.add_resource(Product, "/api/v1/publish/products/<int:product_id>")
    api.add_resource(ProductsOverview, "/api/v1/publish/products/<int:product_id>/overview")
    api.add_resource(
        PublishProduct,
        "/api/v1/publish/products/<int:product_id>/publishers/<string:publisher_id>",
    )



from flask_restful import Resource
from flask import request

from core.managers import presenters_manager
from core.managers.auth_manager import auth_required
from core.model import presenters_node


class PresentersNode(Resource):
    @auth_required("CONFIG_PRESENTERS_NODE_ACCESS")
    def get(self):
        search = request.args.get(key="search", default=None)
        return presenters_node.PresentersNode.get_all_json(search)

    @auth_required("CONFIG_PRESENTERS_NODE_CREATE")
    def post(self):
        return "", presenters_manager.add_presenters_node(request.json)

    @auth_required("CONFIG_PRESENTERS_NODE_UPDATE")
    def put(self, id):
        presenters_manager.update_presenters_node(id, request.json)

    @auth_required("CONFIG_PRESENTERS_NODE_DELETE")
    def delete(self, id):
        return presenters_node.PresentersNode.delete(id)


def initialize(api):
    api.add_resource(PresentersNode, "/api/presenters/nodes", "/api/presenters/node", "/api/presenters/node/<id>")



import io
from flask import request, send_file
from flask_restful import Resource

from core.managers import auth_manager, sse_manager
from core.managers.log_manager import logger
from core.managers.auth_manager import ACLCheck, auth_required
from core.model import news_item, osint_source


class OSINTSourceGroupsAssess(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self):
        return osint_source.OSINTSourceGroup.get_all_json(None, auth_manager.get_user_from_jwt(), True)


class OSINTSourceGroupsList(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self):
        return osint_source.OSINTSourceGroup.get_list_json(user=auth_manager.get_user_from_jwt(), acl_check=True)


class OSINTSourcesList(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self):
        return osint_source.OSINTSource.get_all_with_type()


class ManualOSINTSources(Resource):
    @auth_required(["ASSESS_ACCESS"])
    def get(self):
        return osint_source.OSINTSource.get_all_manual_json(auth_manager.get_user_from_jwt())


class AddNewsItem(Resource):
    @auth_required("ASSESS_CREATE")
    def post(self):
        osint_source_ids = news_item.NewsItemAggregate.add_news_item(request.json)
        sse_manager.news_items_updated()
        sse_manager.remote_access_news_items_updated(osint_source_ids)


class NewsItems(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self):
        user = auth_manager.get_user_from_jwt()

        try:
            filter_keys = ["search", "read", "important", "relevant", "in_analyze", "range", "sort"]
            filter_args = {k: v for k, v in request.args.items() if k in filter_keys}

            group_id = request.args.get("group", osint_source.OSINTSourceGroup.get_default().id)
            offset = int(request.args.get("offset", 0))
            limit = min(int(request.args.get("limit", 50)), 200)
        except Exception as ex:
            logger.log_debug(ex)
            return "", 400

        return news_item.NewsItem.get_by_group_json(group_id, filter_args, offset, limit, user)


class NewsItemAggregates(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self):
        user = auth_manager.get_user_from_jwt()

        try:
            filter_keys = ["search", "read", "important", "relevant", "in_analyze", "range", "sort"]
            filter_args = {k: v for k, v in request.args.items() if k in filter_keys}

            group_id = request.args.get("group", osint_source.OSINTSourceGroup.get_default().id)
            offset = int(request.args.get("offset", 0))
            limit = min(int(request.args.get("limit", 50)), 200)
        except Exception as ex:
            logger.log_debug(ex)
            return "", 400

        return news_item.NewsItemAggregate.get_by_group_json(group_id, filter_args, offset, limit, user)


class NewsItemAggregatesByGroup(Resource):
    # DEPRECATED IN FAVOR OF NewsItemAggregates
    @auth_required("ASSESS_ACCESS")
    def get(self, group_id):
        user = auth_manager.get_user_from_jwt()

        try:
            filter_keys = ["search", "read", "important", "relevant", "in_analyze", "range", "sort"]
            filter_args = {k: v for k, v in request.args.items() if k in filter_keys}

            offset = int(request.args.get("offset", 0))
            limit = min(int(request.args.get("limit", 50)), 200)
        except Exception as ex:
            logger.log_debug(ex)
            return "", 400

        return news_item.NewsItemAggregate.get_by_group_json(group_id, filter_args, offset, limit, user)


class NewsItem(Resource):
    @auth_required("ASSESS_ACCESS", ACLCheck.NEWS_ITEM_ACCESS)
    def get(self, item_id):
        return news_item.NewsItem.get_detail_json(item_id)

    @auth_required("ASSESS_UPDATE", ACLCheck.NEWS_ITEM_MODIFY)
    def put(self, item_id):
        user = auth_manager.get_user_from_jwt()
        response, osint_source_ids, code = news_item.NewsItem.update(item_id, request.json, user.id)
        sse_manager.news_items_updated()
        if len(osint_source_ids) > 0:
            sse_manager.remote_access_news_items_updated(osint_source_ids)
        return response, code

    @auth_required("ASSESS_DELETE", ACLCheck.NEWS_ITEM_MODIFY)
    def delete(self, item_id):
        response, code = news_item.NewsItem.delete(item_id)
        sse_manager.news_items_updated()
        return response, code


class NewsItemAggregate(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self, aggregate_id):
        return news_item.NewsItemAggregate.find(aggregate_id)

    @auth_required("ASSESS_UPDATE")
    def put(self, aggregate_id):
        user = auth_manager.get_user_from_jwt()
        response, osint_source_ids, code = news_item.NewsItemAggregate.update(aggregate_id, request.json, user)
        sse_manager.news_items_updated()
        if len(osint_source_ids) > 0:
            sse_manager.remote_access_news_items_updated(osint_source_ids)
        return response, code

    @auth_required("ASSESS_DELETE")
    def delete(self, aggregate_id):
        user = auth_manager.get_user_from_jwt()
        response, code = news_item.NewsItemAggregate.delete(aggregate_id, user)
        sse_manager.news_items_updated()
        return response, code


class GroupAction(Resource):
    @auth_required("ASSESS_UPDATE")
    def put(self):
        user = auth_manager.get_user_from_jwt()
        response, code = news_item.NewsItemAggregate.group_action(request.json, user)
        sse_manager.news_items_updated()
        return response, code

    @auth_required("ASSESS_UPDATE")
    def delete(self):
        user = auth_manager.get_user_from_jwt()
        response, code = news_item.NewsItemAggregate.group_action_delete(request.json, user)
        sse_manager.news_items_updated()
        return response, code


class DownloadAttachment(Resource):
    def get(self, item_data_id, attribute_id):
        if "jwt" in request.args:
            user = auth_manager.decode_user_from_jwt(request.args["jwt"])
            if user is not None:
                permissions = user.get_permissions()
                if "ASSESS_ACCESS" in permissions:
                    attribute_mapping = news_item.NewsItemDataNewsItemAttribute.find(attribute_id)
                    need_check = attribute_mapping is not None
                    attribute = news_item.NewsItemAttribute.find(attribute_id)
                    if (
                        need_check
                        and item_data_id == attribute_mapping.news_item_data_id
                        and news_item.NewsItemData.allowed_with_acl(
                            attribute_mapping.news_item_data_id,
                            user,
                            False,
                            True,
                            False,
                        )
                    ):
                        logger.store_user_activity(user, "ASSESS_ACCESS", str({"file": attribute.value}))
                        return send_file(
                            io.BytesIO(attribute.binary_data),
                            attachment_filename=attribute.value,
                            mimetype=attribute.binary_mime_type,
                            as_attachment=True,
                        )
                    else:
                        logger.store_auth_error_activity("Unauthorized access attempt to News Item Data")
                else:
                    logger.store_auth_error_activity("Insufficient permissions")
            else:
                logger.store_auth_error_activity("Invalid JWT")
        else:
            logger.store_auth_error_activity("Missing JWT")


def initialize(api):
    api.add_resource(OSINTSourceGroupsAssess, "/api/v1/assess/osint-source-groups")
    api.add_resource(OSINTSourceGroupsList, "/api/v1/assess/osint-source-group-list")
    api.add_resource(OSINTSourcesList, "/api/v1/assess/osint-sources-list")
    api.add_resource(ManualOSINTSources, "/api/v1/assess/manual-osint-sources")
    api.add_resource(AddNewsItem, "/api/v1/assess/news-items")
    api.add_resource(
        NewsItemAggregatesByGroup,
        "/api/v1/assess/news-item-aggregates-by-group/<string:group_id>",
    )
    api.add_resource(
        NewsItemAggregates,
        "/api/v1/assess/news-item-aggregates",
    )
    api.add_resource(
        NewsItems,
        "/api/v1/assess/news-items",
    )
    api.add_resource(NewsItem, "/api/v1/assess/news-items/<int:item_id>")
    api.add_resource(NewsItemAggregate, "/api/v1/assess/news-item-aggregates/<int:aggregate_id>")
    api.add_resource(GroupAction, "/api/v1/assess/news-item-aggregates-group-action")
    api.add_resource(
        DownloadAttachment,
        "/api/v1/assess/news-item-data/<string:item_data_id>/attributes/<int:attribute_id>/file",
    )



from flask_restful import Api

import core.api as core_api


def initialize(app):
    api = Api(app)

    core_api.assess.initialize(api)
    core_api.auth.initialize(api)
    core_api.collectors.initialize(api)
    core_api.isalive.initialize(api)
    core_api.openapi.initialize(api)
    core_api.config.initialize(api)
    core_api.analyze.initialize(api)
    core_api.publish.initialize(api)
    core_api.user.initialize(api)
    core_api.assets.initialize(api)
    core_api.bots.initialize(api)
    core_api.remote.initialize(api)
    core_api.dashboard.initialize(api)



import os
from xml.etree.ElementTree import iterparse
from marshmallow import fields, post_load
from sqlalchemy import orm, func, or_

from core.managers.log_manager import logger
from core.managers.db_manager import db
from shared.schema.attribute import (
    AttributeBaseSchema,
    AttributeEnumSchema,
    AttributeType,
    AttributeValidator,
    AttributePresentationSchema,
)


class NewAttributeEnumSchema(AttributeEnumSchema):
    @post_load
    def make_attribute_enum(self, data, **kwargs):
        return AttributeEnum(**data)


class AttributeEnum(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    index = db.Column(db.Integer)
    value = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    imported = db.Column(db.Boolean, default=False)

    attribute_id = db.Column(db.Integer, db.ForeignKey("attribute.id"))
    attribute = db.relationship("Attribute")

    def __init__(self, id, index, value, description):
        if id is not None and id != -1:
            self.id = id
        else:
            self.id = None

        self.index = index
        self.value = value
        self.description = description

    @classmethod
    def count_for_attribute(cls, attribute_id):
        return cls.query.filter_by(attribute_id=attribute_id).count()

    @classmethod
    def get_all_for_attribute(cls, attribute_id):
        return cls.query.filter_by(attribute_id=attribute_id).order_by(db.asc(AttributeEnum.index)).all()

    @classmethod
    def get_for_attribute(cls, attribute_id, search, offset, limit):
        query = cls.query.filter_by(attribute_id=attribute_id)
        if search:
            search_string = "%" + search + "%"
            query = query.filter(
                or_(
                    AttributeEnum.value.like(search_string),
                    AttributeEnum.description.like(search_string),
                )
            )

        query = query.order_by(db.asc(AttributeEnum.index))

        return query.offset(offset).limit(limit).all(), query.count()

    @classmethod
    def find_by_value(cls, attribute_id, value):
        return cls.query.filter_by(attribute_id=attribute_id).filter(func.lower(AttributeEnum.value) == value.lower()).first()

    @classmethod
    def get_for_attribute_json(cls, attribute_id, search, offset, limit):
        attribute_enums, total_count = cls.get_for_attribute(attribute_id, search, offset, limit)
        attribute_enums_schema = AttributeEnumSchema(many=True)
        return {
            "total_count": total_count,
            "items": attribute_enums_schema.dump(attribute_enums),
        }

    @classmethod
    def delete_for_attribute(cls, attribute_id):
        cls.query.filter_by(attribute_id=attribute_id).delete()
        db.session.commit()

    @classmethod
    def delete_imported_for_attribute(cls, attribute_id):
        cls.query.filter_by(attribute_id=attribute_id, imported=True).delete()
        db.session.commit()

    @classmethod
    def add(cls, attribute_id, data):

        count = 0
        if data["delete_existing"] is True:
            cls.delete_for_attribute(attribute_id)
        else:
            count = cls.count_for_attribute(attribute_id)

        attribute_enums_schema = NewAttributeEnumSchema(many=True)
        attribute_enums = attribute_enums_schema.load(data["items"])

        for attribute_enum in attribute_enums:
            original_attribute_enum = cls.find_by_value(attribute_id, attribute_enum.value)
            if original_attribute_enum is None:
                attribute_enum.attribute_id = attribute_id
                attribute_enum.index = count
                count += 1
                db.session.add(attribute_enum)
            else:
                original_attribute_enum.value = attribute_enum.value
                original_attribute_enum.description = attribute_enum.description

        db.session.commit()

    @classmethod
    def update(cls, enum_id, data):
        attribute_enums_schema = NewAttributeEnumSchema(many=True)
        attribute_enums = attribute_enums_schema.load(data)
        for attribute_enum in attribute_enums:
            original_attribute_enum = cls.query.get(enum_id)
            original_attribute_enum.value = attribute_enum.value
            original_attribute_enum.description = attribute_enum.description
            original_attribute_enum.imported = False

        db.session.commit()

    @classmethod
    def delete(cls, attribute_enum_id):
        db.session.delete(cls.query.get(attribute_enum_id))
        db.session.commit()


class NewAttributeSchema(AttributeBaseSchema):
    attribute_enums = fields.Nested(NewAttributeEnumSchema, many=True)

    @post_load
    def make_attribute(self, data, **kwargs):
        return Attribute(**data)


class Attribute(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    type = db.Column(db.Enum(AttributeType))
    default_value = db.Column(db.String())

    validator = db.Column(db.Enum(AttributeValidator))
    validator_parameter = db.Column(db.String())

    def __init__(
        self,
        id,
        name,
        description,
        type,
        default_value,
        validator,
        validator_parameter,
        attribute_enums,
    ):
        self.id = None
        self.name = name
        self.description = description
        self.type = type
        self.default_value = default_value
        self.validator = validator
        self.validator_parameter = validator_parameter
        self.attribute_enums = attribute_enums
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description

        switcher = {
            AttributeType.STRING: "mdi-form-textbox",
            AttributeType.NUMBER: "mdi-numeric",
            AttributeType.BOOLEAN: "mdi-checkbox-marked-outline",
            AttributeType.RADIO: "mdi-radiobox-marked",
            AttributeType.ENUM: "mdi-format-list-bulleted-type",
            AttributeType.TEXT: "mdi-form-textarea",
            AttributeType.RICH_TEXT: "mdi-format-font",
            AttributeType.DATE: "mdi-calendar-blank-outline",
            AttributeType.TIME: "clock-outline",
            AttributeType.DATE_TIME: "calendar-clock",
            AttributeType.LINK: "mdi-link",
            AttributeType.ATTACHMENT: "mdi-paperclip",
            AttributeType.TLP: "mdi-traffic-light",
            AttributeType.CPE: "mdi-laptop",
            AttributeType.CVE: "mdi-hazard-lights",
            AttributeType.CVSS: "mdi-counter",
        }
        self.tag = switcher.get(self.type, "mdi-textbox")

    @classmethod
    def get_all(cls):
        return cls.query.order_by(Attribute.name).all()

    @classmethod
    def find_by_type(cls, attribute_type):
        return cls.query.filter_by(type=attribute_type).first()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(Attribute.name).like(search_string),
                    func.lower(Attribute.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Attribute.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        attributes, total_count = cls.get(search)
        for attribute in attributes:
            if attribute.type == AttributeType.CPE or attribute.type == AttributeType.CVE:
                attribute.attribute_enums = []
            else:
                attribute.attribute_enums = AttributeEnum.get_all_for_attribute(attribute.id)

        attribute_schema = AttributePresentationSchema(many=True)
        return {"total_count": total_count, "items": attribute_schema.dump(attributes)}

    @classmethod
    def get_enums(cls, attribute):
        if attribute.type == AttributeType.RADIO or attribute.type == AttributeType.ENUM:
            return AttributeEnum.get_all_for_attribute(attribute.id)
        else:
            return []

    @classmethod
    def create_attribute(cls, attribute):
        db.session.add(attribute)
        db.session.commit()

        for attribute_enum in attribute.attribute_enums:
            attribute_enum.attribute_id = attribute.id
            db.session.add(attribute_enum)

        attribute.attribute_enums = []

        db.session.commit()

    @classmethod
    def add_attribute(cls, attribute_data):
        attribute_schema = NewAttributeSchema()
        attribute = attribute_schema.load(attribute_data)
        db.session.add(attribute)
        db.session.commit()

        count = 0
        for attribute_enum in attribute.attribute_enums:
            attribute_enum.attribute_id = attribute.id
            attribute_enum.index = count
            count += 1
            db.session.add(attribute_enum)

        attribute.attribute_enums = []

        db.session.commit()

    @classmethod
    def update(cls, attribute_id, data):
        schema = NewAttributeSchema()
        updated_attribute = schema.load(data)
        attribute = cls.query.get(attribute_id)
        attribute.name = updated_attribute.name
        attribute.description = updated_attribute.description
        attribute.type = updated_attribute.type
        attribute.default_value = updated_attribute.default_value
        attribute.validator = updated_attribute.validator
        attribute.validator_parameter = updated_attribute.validator_parameter
        db.session.commit()

    @classmethod
    def delete_attribute(cls, id):
        attribute = cls.query.get(id)
        AttributeEnum.delete_for_attribute(id)
        db.session.delete(attribute)
        db.session.commit()

    @classmethod
    def load_cve_from_file(cls, file_path):

        attribute = cls.query.filter_by(type=AttributeType.CVE).first()
        AttributeEnum.delete_imported_for_attribute(attribute.id)

        item_count = 0
        block_item_count = 0
        desc = ""
        for event, element in iterparse(file_path, events=("start", "end")):
            if event == "end":
                if element.tag == "{http://cve.mitre.org/cve/downloads/1.0}desc":
                    desc = element.text
                elif element.tag == "{http://cve.mitre.org/cve/downloads/1.0}item":
                    attribute_enum = AttributeEnum(None, item_count, element.attrib["name"], desc)
                    attribute_enum.attribute_id = attribute.id
                    attribute_enum.imported = True
                    db.session.add(attribute_enum)
                    item_count += 1
                    block_item_count += 1
                    element.clear()
                    desc = ""
                    if block_item_count == 1000:
                        logger.log_debug("Processed CVE items: " + str(item_count))
                        block_item_count = 0
                        db.session.commit()

        logger.log_debug("Processed CVE items: " + str(item_count))
        db.session.commit()

    @classmethod
    def load_cpe_from_file(cls, file_path):

        attribute = cls.query.filter_by(type=AttributeType.CPE).first()
        AttributeEnum.delete_imported_for_attribute(attribute.id)

        item_count = 0
        block_item_count = 0
        desc = ""
        for event, element in iterparse(file_path, events=("start", "end")):
            if event == "end":
                logger.log_debug("Element: {}".format(element))
                if element.tag == "{http://cpe.mitre.org/dictionary/2.0}title":
                    desc = element.text
                elif element.tag == "{http://cpe.mitre.org/dictionary/2.0}cpe-item":
                    attribute_enum = AttributeEnum(None, item_count, element.attrib["name"], desc)
                    attribute_enum.attribute_id = attribute.id
                    attribute_enum.imported = True
                    db.session.add(attribute_enum)
                    item_count += 1
                    block_item_count += 1
                    element.clear()
                    desc = ""
                    if block_item_count == 1000:
                        logger.log_debug("Processed CPE items: " + str(item_count))
                        block_item_count = 0
                        db.session.commit()

        logger.log_debug("Processed CPE items: " + str(item_count))
        db.session.commit()

    @classmethod
    def load_dictionaries(cls, dict_type):
        if dict_type == "cve":
            cve_update_file = os.getenv("CVE_UPDATE_FILE")
            if cve_update_file is not None and os.path.exists(cve_update_file):
                Attribute.load_cve_from_file(cve_update_file)

        if dict_type == "cpe":
            cpe_update_file = os.getenv("CPE_UPDATE_FILE")
            if cpe_update_file is not None and os.path.exists(cpe_update_file):
                Attribute.load_cpe_from_file(cpe_update_file)



import uuid
from datetime import datetime

from marshmallow import post_load, fields
from sqlalchemy import orm, func, or_, and_
from sqlalchemy.types import JSON

from core.managers.db_manager import db
from core.managers.log_manager import logger
from core.model.acl_entry import ACLEntry
from core.model.collector import Collector
from core.model.parameter_value import NewParameterValueSchema, ParameterValue
from core.model.word_list import WordList
from shared.schema.acl_entry import ItemType
from shared.schema.osint_source import (
    OSINTSourceSchema,
    OSINTSourceGroupSchema,
    OSINTSourceIdSchema,
    OSINTSourcePresentationSchema,
    OSINTSourceGroupPresentationSchema,
    OSINTSourceGroupIdSchema,
    OSINTSourceGroupSchemaBase,
    OSINTSourceCollectorSchema,
)
from shared.schema.word_list import WordListIdSchema


class NewOSINTSourceSchema(OSINTSourceSchema):
    parameter_values = fields.List(fields.Nested(NewParameterValueSchema))
    word_lists = fields.List(fields.Nested(WordListIdSchema))
    osint_source_groups = fields.List(fields.Nested(OSINTSourceGroupIdSchema))

    @post_load
    def make_osint_source(self, data, **kwargs):
        return OSINTSource(**data)


class OSINTSource(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    collector_id = db.Column(db.String, db.ForeignKey("collector.id"))
    collector = db.relationship("Collector", back_populates="sources")

    parameter_values = db.relationship("ParameterValue", secondary="osint_source_parameter_value", cascade="all")

    word_lists = db.relationship("WordList", secondary="osint_source_word_list")

    modified = db.Column(db.DateTime, default=datetime.now)
    last_collected = db.Column(db.DateTime, default=None)
    last_attempted = db.Column(db.DateTime, default=None)
    state = db.Column(db.SmallInteger, default=0)
    last_error_message = db.Column(db.String, default=None)
    last_data = db.Column(JSON, default=None)

    def __init__(
        self,
        id,
        name,
        description,
        collector_id,
        parameter_values,
        word_lists,
        osint_source_groups,
    ):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.collector_id = collector_id
        self.parameter_values = parameter_values
        self.tag = ""

        self.word_lists = []
        for word_list in word_lists:
            self.word_lists.append(WordList.find(word_list.id))

        self.osint_source_groups = []
        for osint_source_group in osint_source_groups:
            group = OSINTSourceGroup.find(osint_source_group.id)
            if not group.default:
                self.osint_source_groups.append(group)

    @orm.reconstructor
    def reconstruct(self):
        self.tag = "mdi-animation-outline"

    @classmethod
    def find(cls, source_id):
        return cls.query.get(source_id)

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(OSINTSource.name)).all()

    @classmethod
    def get_all_manual(cls, user):
        query = cls.query.join(Collector, OSINTSource.collector_id == Collector.id).filter(Collector.type == "MANUAL_COLLECTOR")

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    OSINTSource.id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.OSINT_SOURCE,
                ),
                and_(
                    OSINTSource.collector_id == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.COLLECTOR,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, False, True, False)

        return query.order_by(db.asc(OSINTSource.name)).all()

    @classmethod
    def get(cls, search=None):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.join(Collector, OSINTSource.collector_id == Collector.id).filter(
                or_(
                    func.lower(OSINTSource.name).like(search_string),
                    func.lower(OSINTSource.description).like(search_string),
                    func.lower(Collector.type).like(search_string),
                )
            )

        return query.order_by(db.asc(OSINTSource.name)).all(), query.count()

    @classmethod
    def get_by_id(cls, id):
        return cls.query.filter_by(id=id).first()

    @classmethod
    def get_all_json(cls, search):
        sources, count = cls.get(search)
        for source in sources:
            source.osint_source_groups = OSINTSourceGroup.get_for_osint_source(source.id)
        sources_schema = OSINTSourcePresentationSchema(many=True)
        return {"total_count": count, "items": sources_schema.dump(sources)}

    @classmethod
    def get_all_with_type(cls):
        query = cls.query

        sources, count = (
            query.join(Collector, OSINTSource.collector_id == Collector.id)
            .with_entities(
                Collector.type.label("collector_type"),
                OSINTSource.id,
                OSINTSource.name,
                OSINTSource.description,
            )
            .order_by(db.asc(OSINTSource.name))
            .all(),
            query.count(),
        )
        sources_schema = OSINTSourceCollectorSchema(many=True)
        return {"total_count": count, "items": sources_schema.dump(sources)}

    @classmethod
    def get_all_manual_json(cls, user):
        sources = cls.get_all_manual(user)
        for source in sources:
            source.osint_source_groups = OSINTSourceGroup.get_for_osint_source(source.id)
        sources_schema = OSINTSourcePresentationSchema(many=True)
        return sources_schema.dump(sources)

    @classmethod
    def get_all_for_collector_json(cls, collector_node, collector_type):
        for collector in collector_node.collectors:
            if collector.type == collector_type:
                sources_schema = OSINTSourceSchema(many=True)
                return sources_schema.dump(collector.sources)

    @classmethod
    def add_new(cls, data):
        new_osint_source_schema = NewOSINTSourceSchema()
        osint_source = new_osint_source_schema.load(data)
        db.session.add(osint_source)

        if len(osint_source.osint_source_groups) > 0:
            for osint_source_group in osint_source.osint_source_groups:
                osint_source_group.osint_sources.append(osint_source)
        else:
            default_group = OSINTSourceGroup.get_default()
            default_group.osint_sources.append(osint_source)

        db.session.commit()

        return osint_source

    @classmethod
    def import_new(cls, osint_source, collector):
        parameter_values = []
        for parameter_value in osint_source.parameter_values:
            for parameter in collector.parameters:
                if parameter.key == parameter_value.parameter.key:
                    new_parameter_value = ParameterValue(parameter_value.value, parameter)
                    parameter_values.append(new_parameter_value)
                    break

        news_osint_source = OSINTSource(
            "",
            osint_source.name,
            osint_source.description,
            collector.id,
            parameter_values,
            [],
            [],
        )

        db.session.add(news_osint_source)
        default_group = OSINTSourceGroup.get_default()
        default_group.osint_sources.append(news_osint_source)
        db.session.commit()

    @classmethod
    def delete(cls, osint_source_id):
        osint_source = cls.query.get(osint_source_id)
        db.session.delete(osint_source)
        db.session.commit()

    @classmethod
    def update(cls, osint_source_id, data):
        new_osint_source_schema = NewOSINTSourceSchema()
        updated_osint_source = new_osint_source_schema.load(data)
        osint_source = cls.query.get(osint_source_id)
        osint_source.name = updated_osint_source.name
        osint_source.description = updated_osint_source.description

        for value in osint_source.parameter_values:
            for updated_value in updated_osint_source.parameter_values:
                if value.parameter_id == updated_value.parameter_id:
                    value.value = updated_value.value

        osint_source.word_lists = updated_osint_source.word_lists

        current_groups = OSINTSourceGroup.get_for_osint_source(osint_source_id)
        default_group = None
        for group in current_groups:
            if group.default:
                default_group = group

            for source in group.osint_sources:
                if source.id == osint_source_id:
                    group.osint_sources.remove(source)
                    break

        if len(updated_osint_source.osint_source_groups) > 0:
            for osint_source_group in updated_osint_source.osint_source_groups:
                osint_source_group.osint_sources.append(osint_source)
        else:
            default_group = OSINTSourceGroup.get_default()
            default_group.osint_sources.append(osint_source)

        db.session.commit()

        return osint_source, default_group

    def update_status(self, status_schema):
        # if not collected, do not change last collected timestamp
        if status_schema.last_collected:
            self.last_collected = status_schema.last_collected

        # if not attempted, do not change last collected timestamp
        if status_schema.last_attempted:
            self.last_attempted = status_schema.last_attempted

        self.last_error_message = status_schema.last_error_message
        self.last_data = status_schema.last_data


class OSINTSourceParameterValue(db.Model):
    osint_source_id = db.Column(db.String, db.ForeignKey("osint_source.id"), primary_key=True)
    parameter_value_id = db.Column(db.Integer, db.ForeignKey("parameter_value.id"), primary_key=True)


class OSINTSourceWordList(db.Model):
    osint_source_id = db.Column(db.String, db.ForeignKey("osint_source.id"), primary_key=True)
    word_list_id = db.Column(db.Integer, db.ForeignKey("word_list.id"), primary_key=True)


class NewOSINTSourceGroupSchema(OSINTSourceGroupSchema):
    osint_sources = fields.Nested(OSINTSourceIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return OSINTSourceGroup(**data)


class OSINTSourceGroup(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    default = db.Column(db.Boolean(), default=False)

    osint_sources = db.relationship("OSINTSource", secondary="osint_source_group_osint_source")

    def __init__(self, id, name, description, default, osint_sources):
        self.id = id or str(uuid.uuid4())
        self.name = name
        self.description = description
        self.default = default
        self.tag = ""
        self.osint_sources = [OSINTSource.find(osint_source.id) for osint_source in osint_sources]

    @orm.reconstructor
    def reconstruct(self):
        self.tag = "mdi-folder-multiple"

    @classmethod
    def find(cls, group_id):
        return cls.query.get(group_id)

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(OSINTSourceGroup.name)).all()

    @classmethod
    def get_for_osint_source(cls, osint_source_id):
        return cls.query.join(
            OSINTSourceGroupOSINTSource,
            and_(
                OSINTSourceGroupOSINTSource.osint_source_id == osint_source_id,
                OSINTSourceGroup.id == OSINTSourceGroupOSINTSource.osint_source_group_id,
            ),
        ).all()

    @classmethod
    def get_default(cls):
        return cls.query.filter(OSINTSourceGroup.default).first()

    @classmethod
    def allowed_with_acl(cls, group_id, user, see: bool, access: bool, modify: bool) -> bool:

        query = db.session.query(OSINTSourceGroup.id).distinct().group_by(OSINTSourceGroup.id).filter(OSINTSourceGroup.id == group_id)

        query = query.outerjoin(
            ACLEntry,
            and_(
                OSINTSourceGroup.id == ACLEntry.item_id,
                ACLEntry.item_type == ItemType.OSINT_SOURCE_GROUP,
            ),
        )
        query = ACLEntry.apply_query(query, user, see, access, modify)
        acl_check_result = query.scalar() is not None
        logger.log_debug(f"ACL Check for {group_id} results: {acl_check_result}")

        return acl_check_result

    @classmethod
    def get(cls, search, user, acl_check):
        query = cls.query.distinct().group_by(OSINTSourceGroup.id)

        if acl_check:
            query = query.outerjoin(
                ACLEntry, and_(OSINTSourceGroup.id == ACLEntry.item_id, ACLEntry.item_type == ItemType.OSINT_SOURCE_GROUP)
            )
            query = ACLEntry.apply_query(query, user, True, False, False)

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(OSINTSourceGroup.name).like(search_string),
                    func.lower(OSINTSourceGroup.description).like(search_string),
                )
            )

        return (
            query.all(),
            query.count(),
        )

    @classmethod
    def get_all_json(cls, search, user, acl_check):
        groups, count = cls.get(search, user, acl_check)
        group_schema = OSINTSourceGroupPresentationSchema(many=True)
        return {"total_count": count, "items": group_schema.dump(groups)}

    @classmethod
    def get_list_json(cls, user, acl_check):
        groups, count = cls.get(None, user, acl_check)
        group_schema = OSINTSourceGroupSchemaBase(many=True)
        return {"total_count": count, "items": group_schema.dump(groups)}

    @classmethod
    def get_all_with_source(cls, osint_source_id):
        all_groups = cls.get_all()
        groups = []
        for group in all_groups:
            for source in group.osint_sources:
                if source.id == osint_source_id:
                    groups.append(group)
                    break

        return groups

    @classmethod
    def add(cls, data):
        new_osint_source_group_schema = NewOSINTSourceGroupSchema()
        osint_source_group = new_osint_source_group_schema.load(data)
        db.session.add(osint_source_group)
        db.session.commit()

    @classmethod
    def create(cls, group_id, name, description, default=False):
        osint_source_group = cls.find(group_id)
        if osint_source_group is None:
            osint_source_group = OSINTSourceGroup(group_id, name, description, default, [])
            db.session.add(osint_source_group)
        else:
            osint_source_group.name = name
            osint_source_group.description = description
        db.session.commit()

    @classmethod
    def delete(cls, osint_source_group_id):
        osint_source_group = cls.query.get(osint_source_group_id)
        if osint_source_group.default is not False:
            return {"message": "could_not_delete_default_group"}, 400
        db.session.delete(osint_source_group)
        db.session.commit()
        return "", 200

    @classmethod
    def update(cls, osint_source_group_id, data):
        new_osint_source_group_schema = NewOSINTSourceGroupSchema()
        updated_osint_source_group = new_osint_source_group_schema.load(data)
        osint_source_group = cls.query.get(osint_source_group_id)
        if osint_source_group.default:
            return None, {"message": "could_not_modify_default_group"}, 400

        osint_source_group.name = updated_osint_source_group.name
        osint_source_group.description = updated_osint_source_group.description
        osint_source_group.osint_sources = updated_osint_source_group.osint_sources

        sources_in_default_group = set()
        for source in osint_source_group.osint_sources:
            current_groups = OSINTSourceGroup.get_for_osint_source(source.id)
            for current_group in current_groups:
                if current_group.default:
                    current_group.osint_sources.remove(source)
                    sources_in_default_group.add(source)
                    break

        db.session.commit()

        return sources_in_default_group, "", 200


class OSINTSourceGroupOSINTSource(db.Model):
    osint_source_group_id = db.Column(db.String, db.ForeignKey("osint_source_group.id", ondelete="CASCADE"), primary_key=True)
    osint_source_id = db.Column(db.String, db.ForeignKey("osint_source.id", ondelete="CASCADE"), primary_key=True)



from marshmallow import post_load, fields
from datetime import datetime
import uuid as uuid_generator
from sqlalchemy import orm, or_, func, text, and_
from sqlalchemy.sql.expression import cast
import sqlalchemy

from core.managers.db_manager import db
from core.managers.log_manager import logger
from core.model.news_item import NewsItemAggregate
from core.model.report_item_type import AttributeGroupItem
from core.model.report_item_type import ReportItemType
from core.model.acl_entry import ACLEntry
from shared.schema.acl_entry import ItemType
from shared.schema.attribute import AttributeType
from shared.schema.news_item import NewsItemAggregateIdSchema, NewsItemAggregateSchema
from shared.schema.report_item import (
    ReportItemAttributeBaseSchema,
    ReportItemBaseSchema,
    ReportItemIdSchema,
    RemoteReportItemSchema,
)
from shared.schema.report_item import (
    ReportItemRemoteSchema,
    ReportItemSchema,
    ReportItemPresentationSchema,
)


class NewReportItemAttributeSchema(ReportItemAttributeBaseSchema):
    @post_load
    def make_report_item_attribute(self, data, **kwargs):
        return ReportItemAttribute(**data)


class ReportItemAttribute(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    value = db.Column(db.String(), nullable=False)
    binary_mime_type = db.Column(db.String())
    binary_data = orm.deferred(db.Column(db.LargeBinary))
    binary_size = db.Column(db.Integer)
    binary_description = db.Column(db.String())
    created = db.Column(db.DateTime, default=datetime.now)
    last_updated = db.Column(db.DateTime, default=datetime.now)

    version = db.Column(db.Integer, default=1)
    current = db.Column(db.Boolean, default=True)

    attribute_group_item_id = db.Column(db.Integer, db.ForeignKey("attribute_group_item.id"))
    attribute_group_item = db.relationship("AttributeGroupItem")
    attribute_group_item_title = db.Column(db.String)

    report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"), nullable=True)
    report_item = db.relationship("ReportItem")

    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), nullable=True)
    user = db.relationship("User")

    def __init__(
        self,
        id,
        value,
        binary_mime_type,
        binary_size,
        binary_description,
        attribute_group_item_id,
        attribute_group_item_title,
    ):
        self.id = None
        self.value = value
        self.binary_mime_type = binary_mime_type
        self.binary_size = binary_size
        self.binary_description = binary_description
        self.attribute_group_item_id = attribute_group_item_id
        self.attribute_group_item_title = attribute_group_item_title

    @classmethod
    def find(cls, attribute_id):
        return cls.query.get(attribute_id)

    @staticmethod
    def sort(report_item_attribute):
        return report_item_attribute.last_updated


class NewReportItemSchema(ReportItemBaseSchema):
    news_item_aggregates = fields.Nested(NewsItemAggregateIdSchema, many=True, load_default=[])
    remote_report_items = fields.Nested(ReportItemIdSchema, many=True, load_default=[])
    attributes = fields.Nested(NewReportItemAttributeSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return ReportItem(**data)


class ReportItemRemoteReportItem(db.Model):
    report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"), primary_key=True)
    remote_report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"), primary_key=True)


class ReportItem(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    uuid = db.Column(db.String(64))

    title = db.Column(db.String())
    title_prefix = db.Column(db.String())

    created = db.Column(db.DateTime, default=datetime.now)
    last_updated = db.Column(db.DateTime, default=datetime.now)
    completed = db.Column(db.Boolean, default=False)

    user_id = db.Column(db.Integer, db.ForeignKey("user.id"), nullable=True)
    user = db.relationship("User")
    remote_user = db.Column(db.String())

    report_item_type_id = db.Column(db.Integer, db.ForeignKey("report_item_type.id"), nullable=True)
    report_item_type = db.relationship("ReportItemType")

    news_item_aggregates = db.relationship("NewsItemAggregate", secondary="report_item_news_item_aggregate")

    remote_report_items = db.relationship(
        "ReportItem",
        secondary="report_item_remote_report_item",
        primaryjoin=ReportItemRemoteReportItem.report_item_id == id,
        secondaryjoin=ReportItemRemoteReportItem.remote_report_item_id == id,
    )

    attributes = db.relationship(
        "ReportItemAttribute",
        back_populates="report_item",
        cascade="all, delete-orphan",
    )

    report_item_cpes = db.relationship("ReportItemCpe", cascade="all, delete-orphan", back_populates="report_item")

    def __init__(
        self,
        id,
        uuid,
        title,
        title_prefix,
        report_item_type_id,
        news_item_aggregates,
        remote_report_items,
        attributes,
        completed,
    ):

        self.id = id

        self.uuid = str(uuid_generator.uuid4()) if uuid is None else uuid
        self.title = title
        self.title_prefix = title_prefix
        self.report_item_type_id = report_item_type_id
        self.attributes = attributes
        self.completed = completed
        self.report_item_cpes = []
        self.subtitle = ""
        self.tag = ""

        self.news_item_aggregates = [NewsItemAggregate.find(news_item_aggregate.id) for news_item_aggregate in news_item_aggregates]

        self.remote_report_items = [ReportItem.find(remote_report_item.id) for remote_report_item in remote_report_items]

    @orm.reconstructor
    def reconstruct(self):
        self.subtitle = ""
        self.tag = "mdi-file-table-outline"
        self.attributes.sort(key=ReportItemAttribute.sort)

    @classmethod
    def count_all(cls, is_completed):
        return cls.query.filter_by(completed=is_completed).count()

    @classmethod
    def find(cls, report_item_id):
        return cls.query.get(report_item_id)

    @classmethod
    def find_by_uuid(cls, report_item_uuid):
        return cls.query.filter_by(uuid=report_item_uuid)

    @classmethod
    def allowed_with_acl(cls, report_item_id, user, see, access, modify):

        query = db.session.query(ReportItem.id).distinct().group_by(ReportItem.id).filter(ReportItem.id == report_item_id)

        query = query.outerjoin(
            ACLEntry,
            or_(
                and_(
                    ReportItem.uuid == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.REPORT_ITEM,
                ),
                and_(
                    cast(ReportItem.report_item_type_id, sqlalchemy.String) == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.REPORT_ITEM_TYPE,
                ),
            ),
        )

        query = ACLEntry.apply_query(query, user, see, access, modify)

        return query.scalar() is not None

    @classmethod
    def get_for_sync(cls, last_synced, report_item_types):
        report_item_type_ids = {report_item_type.id for report_item_type in report_item_types}

        last_sync_time = datetime.now()

        query = cls.query.filter(
            ReportItem.last_updated >= last_synced,
            ReportItem.last_updated <= last_sync_time,
            ReportItem.report_item_type_id.in_(report_item_type_ids),
        )

        report_items = query.all()

        for report_item in report_items:
            for attribute in report_item.attributes:
                attribute.attribute_group_item_title = attribute.attribute_group_item.title

        report_item_remote_schema = ReportItemRemoteSchema(many=True)
        items = report_item_remote_schema.dump(report_items)

        return items, last_sync_time

    @classmethod
    def get(cls, group, filter: dict, offset: int, limit: int, user, acl_check: bool):
        query = cls.query

        if acl_check:
            query = query.outerjoin(
                ACLEntry,
                and_(
                    ReportItem.uuid == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.REPORT_ITEM,
                ),
            )
            query = ACLEntry.apply_query(query, user, True, False, False)

        if group:
            query = cls.query.filter(ReportItem.remote_user == group)

        if "search" in filter and filter["search"] != "":
            search_string = f"%{filter['search'].lower()}%"
            query = query.join(ReportItemAttribute, ReportItem.id == ReportItemAttribute.report_item_id).filter(
                or_(
                    func.lower(ReportItemAttribute.value).like(search_string),
                    func.lower(ReportItem.title).like(search_string),
                    func.lower(ReportItem.title_prefix).like(search_string),
                )
            )

        if "completed" in filter and filter["completed"].lower() == "true":
            query = query.filter(ReportItem.completed)

        if "incompleted" in filter and filter["incompleted"].lower() == "true":
            query = query.filter(ReportItem.completed is False)

        if "range" in filter and filter["range"] != "ALL":
            date_limit = datetime.now()
            if filter["range"] == "TODAY":
                date_limit = date_limit.replace(hour=0, minute=0, second=0, microsecond=0)

            if filter["range"] == "WEEK":
                date_limit = date_limit.replace(
                    day=date_limit.day - date_limit.weekday(),
                    hour=0,
                    minute=0,
                    second=0,
                    microsecond=0,
                )

            if filter["range"] == "MONTH":
                date_limit = date_limit.replace(day=1, hour=0, minute=0, second=0, microsecond=0)

            query = query.filter(ReportItem.created >= date_limit)

        if "sort" in filter:
            if filter["sort"] == "DATE_DESC":
                query = query.order_by(db.desc(ReportItem.created))

            elif filter["sort"] == "DATE_ASC":
                query = query.order_by(db.asc(ReportItem.created))
        return query.offset(offset).limit(limit).all(), query.count()

    @classmethod
    def identical(cls, uuid):
        return db.session.query(db.exists().where(ReportItem.uuid == uuid)).scalar()

    @classmethod
    def get_by_cpe(cls, cpes):
        if len(cpes) <= 0:
            return []
        query_string = "SELECT DISTINCT report_item_id FROM report_item_cpe WHERE value LIKE ANY(:cpes) OR {}"
        params = {"cpes": cpes}

        inner_query = ""
        for i in range(len(cpes)):
            if i > 0:
                inner_query += " OR "
            param = f"cpe{str(i)}"
            inner_query += f":{param} LIKE value"
            params[param] = cpes[i]

        result = db.engine.execute(text(query_string.format(inner_query)), params)

        return [row[0] for row in result]

    @classmethod
    def get_json(cls, group, filter, offset, limit, user):
        results, count = cls.get(group, filter, offset, limit, user, True)
        logger.log_debug(f"Found {count} report items with filter {filter}")
        report_items_schema = ReportItemPresentationSchema(many=True)
        return {"total_count": count, "items": report_items_schema.dump(results)}

    @classmethod
    def get_detail_json(cls, id):
        report_item = cls.query.get(id)
        report_item_schema = ReportItemSchema()
        return report_item_schema.dump(report_item)

    @classmethod
    def get_groups(cls):
        result = (
            db.session.query(ReportItem.remote_user)
            .distinct()
            .group_by(ReportItem.remote_user)
            .filter(ReportItem.remote_user is not None)
            .all()
        )
        groups = {row[0] for row in result}
        return list(groups)

    @classmethod
    def add_report_item(cls, report_item_data, user):
        report_item_schema = NewReportItemSchema()
        report_item = report_item_schema.load(report_item_data)

        if not ReportItemType.allowed_with_acl(report_item.report_item_type_id, user, False, False, True):
            return "Unauthorized access to report item type", 401

        report_item.user_id = user.id
        for attribute in report_item.attributes:
            attribute.user_id = user.id

        report_item.update_cpes()

        db.session.add(report_item)
        db.session.commit()

        return report_item, 200

    @classmethod
    def add_remote_report_items(cls, report_item_data, remote_node_name):
        report_item_schema = NewReportItemSchema(many=True)
        report_items = report_item_schema.load(report_item_data)

        for report_item in report_items:
            original_report_item = cls.find_by_uuid(report_item.uuid)
            if original_report_item is None:
                report_item.remote_user = remote_node_name
                db.session.add(report_item)
            else:
                original_report_item.title = report_item.title
                original_report_item.title_prefix = report_item.title_prefix
                original_report_item.completed = report_item.completed
                original_report_item.last_updated = datetime.now()
                original_report_item.attributes = report_item.attributes

        db.session.commit()

    @classmethod
    def update_report_item(cls, id, data, user):
        modified = False
        new_attribute = None
        report_item = cls.query.get(id)
        if report_item is not None:
            if "update" in data:
                if "title" in data and report_item.title != data["title"]:
                    modified = True
                    report_item.title = data["title"]
                    data["title"] = ""

                if "title_prefix" in data and report_item.title_prefix != data["title_prefix"]:
                    modified = True
                    report_item.title_prefix = data["title_prefix"]
                    data["title_prefix"] = ""

                if "completed" in data and report_item.completed != data["completed"]:
                    modified = True
                    report_item.completed = data["completed"]
                    data["completed"] = ""

                if "attribute_id" in data:
                    for attribute in report_item.attributes:
                        if attribute.id == data["attribute_id"] and attribute.value != data["attribute_value"]:
                            modified = True
                            attribute.value = data["attribute_value"]
                            data["attribute_value"] = ""
                            attribute.user = user
                            attribute.last_updated = datetime.now()
                            break

            if "add" in data:
                if "attribute_id" in data:
                    modified = True
                    new_attribute = ReportItemAttribute(None, "", None, 0, None, data["attribute_group_item_id"], None)
                    new_attribute.user = user
                    report_item.attributes.append(new_attribute)

                if "aggregate_ids" in data:
                    modified = True
                    for aggregate_id in data["aggregate_ids"]:
                        aggregate = NewsItemAggregate.find(aggregate_id)
                        report_item.news_item_aggregates.append(aggregate)

                if "remote_report_item_ids" in data:
                    modified = True
                    for remote_report_item_id in data["remote_report_item_ids"]:
                        remote_report_item = ReportItem.find(remote_report_item_id)
                        report_item.remote_report_items.append(remote_report_item)

            if "delete" in data:
                if "attribute_id" in data:
                    attribute_to_delete = None
                    for attribute in report_item.attributes:
                        if attribute.id == data["attribute_id"]:
                            attribute_to_delete = attribute
                            break

                    if attribute_to_delete is not None:
                        modified = True
                        report_item.attributes.remove(attribute_to_delete)

                if "aggregate_id" in data:
                    aggregate_to_delete = None
                    for aggregate in report_item.news_item_aggregates:
                        if aggregate.id == data["aggregate_id"]:
                            aggregate_to_delete = aggregate
                            break

                    if aggregate_to_delete is not None:
                        modified = True
                        report_item.news_item_aggregates.remove(aggregate_to_delete)

                if "remote_report_item_id" in data:
                    remote_report_item_to_delete = None
                    for remote_report_item in report_item.remote_report_items:
                        if remote_report_item.id == data["remote_report_item_id"]:
                            remote_report_item_to_delete = remote_report_item
                            break

                    if remote_report_item_to_delete is not None:
                        modified = True
                        report_item.remote_report_items.remove(remote_report_item_to_delete)

            if modified:
                report_item.last_updated = datetime.now()
                data["user_id"] = user.id
                data["report_item_id"] = int(id)
                report_item.update_cpes()

            db.session.commit()

            if new_attribute is not None:
                data["attribute_id"] = new_attribute.id

        return modified, data

    @classmethod
    def get_updated_data(cls, id, data):
        report_item = cls.query.get(id)
        if report_item is not None:
            if "update" in data:
                if "title" in data:
                    data["title"] = report_item.title

                if "title_prefix" in data:
                    data["title_prefix"] = report_item.title_prefix

                if "completed" in data:
                    data["completed"] = report_item.completed

                if "attribute_id" in data:
                    for attribute in report_item.attributes:
                        if attribute.id == data["attribute_id"]:
                            data["attribute_value"] = attribute.value
                            data["attribute_last_updated"] = attribute.last_updated.strftime("%d.%m.%Y - %H:%M")
                            data["attribute_user"] = attribute.user.name
                            break

            if "add" in data:
                if "aggregate_ids" in data:
                    schema = NewsItemAggregateSchema()
                    data["news_item_aggregates"] = []
                    for aggregate_id in data["aggregate_ids"]:
                        aggregate = NewsItemAggregate.find(aggregate_id)
                        data["news_item_aggregates"].append(schema.dump(aggregate))

                if "remote_report_item_ids" in data:
                    schema = RemoteReportItemSchema()
                    data["remote_report_items"] = []
                    for remote_report_item_id in data["remote_report_item_ids"]:
                        remote_report_item = ReportItem.find(remote_report_item_id)
                        data["remote_report_items"].append(schema.dump(remote_report_item))

                if "attribute_id" in data:
                    for attribute in report_item.attributes:
                        if attribute.id == data["attribute_id"]:
                            data["attribute_value"] = attribute.value
                            data["binary_mime_type"] = attribute.binary_mime_type
                            data["binary_size"] = attribute.binary_size
                            data["binary_description"] = attribute.binary_description
                            data["attribute_last_updated"] = attribute.last_updated.strftime("%d.%m.%Y - %H:%M")
                            data["attribute_user"] = attribute.user.name
                            break

        return data

    @classmethod
    def add_attachment(cls, id, attribute_group_item_id, user, file, description):
        report_item = cls.query.get(id)
        file_data = file.read()
        new_attribute = ReportItemAttribute(
            None,
            file.filename,
            file.mimetype,
            len(file_data),
            description,
            attribute_group_item_id,
            None,
        )
        new_attribute.user = user
        new_attribute.binary_data = file_data
        report_item.attributes.append(new_attribute)

        report_item.last_updated = datetime.now()

        data = dict()
        data["add"] = True
        data["user_id"] = user.id
        data["report_item_id"] = int(id)
        data["attribute_id"] = new_attribute.id

        db.session.commit()

        return data

    @classmethod
    def remove_attachment(cls, id, attribute_id, user):
        report_item = cls.query.get(id)
        attribute_to_delete = None
        for attribute in report_item.attributes:
            if attribute.id == attribute_id:
                attribute_to_delete = attribute
                break

        if attribute_to_delete is not None:
            report_item.attributes.remove(attribute_to_delete)

        report_item.last_updated = datetime.now()

        data = dict()
        data["delete"] = True
        data["user_id"] = user.id
        data["report_item_id"] = int(id)
        data["attribute_id"] = attribute_id

        db.session.commit()

        return data

    @classmethod
    def delete_report_item(cls, id):
        report_item = cls.query.get(id)
        if report_item is not None:
            db.session.delete(report_item)
            db.session.commit()
            return "success", 200

    def update_cpes(self):
        self.report_item_cpes = []
        if self.completed is True:
            for attribute in self.attributes:
                attribute_group = AttributeGroupItem.find(attribute.attribute_group_item_id)
                if attribute_group.attribute.type == AttributeType.CPE:
                    self.report_item_cpes.append(ReportItemCpe(attribute.value))


class ReportItemCpe(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    value = db.Column(db.String())

    report_item_id = db.Column(db.Integer, db.ForeignKey("report_item.id"))
    report_item = db.relationship("ReportItem")

    def __init__(self, value):
        self.id = None
        self.value = value
